#ifndef _I3STATBX_
  #define _I3STATBX_
/*******************************************************************************
* FILE NAME: i3statbx.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     I3statbx - This class creates and manages the three-state check box      *
*                control window.                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
* $Log:   R:/IBMCLASS/IBASECTL/VCS/I3STATBX.HPV  $
//
//   Rev 1.8   25 Oct 1992 17:05:00   nunn
//changed library name to ICLUI
//
//   Rev 1.7   25 Oct 1992 10:47:52   boezeman
//Added documentation and converted file to skeleton.hpp format

      Rev 1.6   13 Oct 1992 21:17:36   tsuji
   Correct comments around log directive.

      Rev 1.4   13 Oct 1992 20:55:52   tsuji
   Correct log directive, remove IBM Confidential classification.

      Rev 1.3   13 Oct 1992 19:02:08   tsuji
   Remove no-op virtual destructor, add Inherited typedef
*******************************************************************************/
#ifndef _ISETBUT_
  #include <isetbut.hpp>
#endif

// Forward declarations for other classes:
class I3StateCheckBox;  /* s3bx */
class IRectangle;
class ISize;
class IWindowHandle;
#ifndef _IBITFLAG_
  #include <ibitflag.hpp>
#endif

class I3StateCheckBox : public ISettingButton {
/*******************************************************************************
* This class creates and manages the three-state box control window.           *
*                                                                              *
* To use this class, create an instance of this class as follows:              *
*   I3StateCheckBox s3bxChoice(ID_CHOICE, this, IRectangle(10,10,30,12));      *
*   s3bxChoice.setText("Sweet and Sour");                                      *
* User action or invoking an operation on the three-state check box            *
* will cause an IControlEvent.                                                 *
* Use the id() and notifyCode() operations in IControlEvent to process the     *
* event(e.g. a notification code of BN_CLICKED or BN_DBLCLICKED).              *
*                                                                              *
* EXAMPLE:                                                                     *
*   <sample code>                                                              *
*******************************************************************************/
typedef ISettingButton
  Inherited;
public:
INESTEDBITFLAGCLASSDEF3(Style, I3StateCheckBox,
                                 IWindow, IControl, IButton);
                                // style class definition
static const Style
  autoSelect;
static const Style& defStyle;
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 3 ways to construct instances of this class:                       |
|   1. default                                                                 |
|   2. From a Dialog Template                                                  |
|   3. From a Window                                                           |
------------------------------------------------------------------------------*/
  I3StateCheckBox(unsigned long Id,
                  const IWindow* parent,
                  const IWindow* owner,
                  const IRectangle& initial,
                  Style style = defStyle);

  I3StateCheckBox(unsigned long ulId,
                  const IWindow* parentDialog);

  I3StateCheckBox(IWindowHandle handle);

/*-------------------------------- STYLES --------------------------------------
| These function provide means of getting and setting the default style        |
| attributes of instances of this class:                                       |
|   defaultStyle    - returns the three state control default style setting    |
|   setDefaultStyle - sets the three state control default style               |
------------------------------------------------------------------------------*/
static Style
   defaultStyle();
static void
   setDefaultStyle(Style style);

/*-------------------------------- OVERLOADED AUTOSELECT------------------------
| These operations are overloaded since the actual style setting is different. |
|  enableAutoSelect  - sets/removes the three state control to/from the        |
|                       autoselect style                                       |
|  disableAutoSelect - removes the autoselect style from the three state control|
|  isAutoSelect      - returns true is the three state control has the         |
|                      autoselect style set else it returns false              |
------------------------------------------------------------------------------*/
virtual void
  enableAutoSelect(),
  disableAutoSelect();
virtual Boolean
  isAutoSelect() const;

/*-------------------------------- HALFTONE OPERATIONS -------------------------
|  selectHalftoned   - Selects a halftoned three state check box.              |
|  isHalftoned       - Returns true if the three state control has been        |
|                      halftoned selected.                                     |
------------------------------------------------------------------------------*/
void
  selectHalftoned();
Boolean
  isHalftoned() const;

protected:
/*----------------------------- LAYOUT SIZE ------------------------------------
| calcMinSize - Returns the minimum size that this three state control should be|
------------------------------------------------------------------------------*/
virtual ISize
  calcMinSize();

static Style
  classDefaultStyle;

private:
/*--------------------------------- PRIVATE ----------------------------------*/
  I3StateCheckBox(const I3StateCheckBox&);
//I3StateCheckBox& operator=(const I3StateCheckBox&);

static const Style
  threeState;
};  // I3StateCheckBox

INESTEDBITFLAGCLASSFUNCS(Style, I3StateCheckBox);
                                  // global style functions
#endif  /* _I3STATBX_ */
