/* Copyright (c) IBM Corp. 1992 */
 

template < class Element >
INumber IASortedSet < Element >::
numberOfOccurrences (Element const& element) const
{ if (contains (element))
    return 1;
  else
    return 0;
}

template < class Element >
Boolean IASortedSet < Element >::
locateNext (Element const&, ICursor& cursor) const
{ cursor.invalidate ();
  return False;
}

template < class Element >
INumber IASortedSet < Element >::
removeAllOccurrences (Element const& element)
{ if (remove (element))
    return 1;
  else
    return 0;
}

template < class Element >
INumber IASortedSet < Element >::
numberOfDifferentElements () const
{ return numberOfElements ();
}

template < class Element >
Boolean IASortedSet < Element >::
setToNextDifferentElement (ICursor& cursor) const
{ return setToNext (cursor);
}

template < class Element >
Boolean IASortedSet < Element >::
operator == (IASortedSet < Element > const& collection) const
{ return this->containsAllFrom (collection) &&
	 collection.containsAllFrom (*this);
}

template < class Element >
Boolean IASortedSet < Element >::
operator != (IASortedSet < Element > const& collection) const
{ return ! operator == (collection);
}

template < class Element >
void IASortedSet < Element >::
unionWith (IASortedSet < Element > const& collection)
{ addAllFrom (collection);
}

template < class Element >
void IASortedSet < Element >::
intersectionWith (IASortedSet < Element > const& collection)
{ IAEqualityCollection < Element > const* env = &collection;
  removeAll (IAEqualityCollection < Element >::isNotContained, &env);
}

template < class Element >
void IASortedSet < Element >::
differenceWith (IASortedSet < Element > const& collection)
{ IAEqualityCollection < Element > const* env = &collection;
  removeAll (IAEqualityCollection < Element >::isContained, &env);
}

template < class Element >
void IASortedSet < Element >::
addUnion (IASortedSet < Element > const& collection1,
          IASortedSet < Element > const& collection2)
{ addAllFrom (collection1);
  addAllFrom (collection2);
}

template < class Element >
void IASortedSet < Element >::
addIntersection (IASortedSet < Element > const& collection1,
                 IASortedSet < Element > const& collection2)
{ ICursor *cursor = collection1.newCursor ();
  forCursor (*cursor) {
    if (collection2.contains (collection1.elementAt (*cursor)))
      add (collection1.elementAt (*cursor));
  }
  delete cursor;
}

template < class Element >
void IASortedSet < Element >::
addDifference (IASortedSet < Element > const& collection1,
               IASortedSet < Element > const& collection2)
{ ICursor *cursor = collection1.newCursor ();
  forCursor (*cursor) {
    if (! collection2.contains (collection1.elementAt (*cursor)))
      add (collection1.elementAt (*cursor));
  }
  delete cursor;
}

template < class Element >
long IASortedSet < Element >::
compare (IASortedSet < Element > const& collection,
         long (*comparisonFunction) (Element const&, Element const&)) const
{ long result = 0;
  ICursor *thisCursor = newCursor ();
  ICursor *collectionCursor = collection.newCursor ();

  for (thisCursor->setToFirst (), collectionCursor->setToFirst ();
       result == 0 && thisCursor->isValid () && collectionCursor->isValid ();
       thisCursor->setToNext (), collectionCursor->setToNext ()) {
    result = (*comparisonFunction) (this->elementAt (*thisCursor),
				      collection.elementAt (*collectionCursor));
  }

  if (result == 0) {
    if (thisCursor->isValid () && ! collectionCursor->isValid ())
      result = 1;
    else if (! thisCursor->isValid () && collectionCursor->isValid ())
      result = -1;
    else {
    }
  }
  delete thisCursor;
  delete collectionCursor;

  return result;
}

