/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _IBAGKSS_H
#define _IBAGKSS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element >
class IBKSElm {
public:
  Element elem;
  INumber occurrences;
  IBKSElm (Element const& e) : elem (e), occurrences (1) {}
  IBKSElm (Element const& e, INumber occurrences)
  : elem (e), occurrences (occurrences) {}

  friend inline Element const& key (IBKSElm < Element > const& e)
  { return e.elem; }
};

template < class Element, class ElementOps, class Base >
class IWBagOnKSSet {
  Base ivBase;
  INumber ivNumberOfElements;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
    INumber occurrence;
    friend class IWBagOnKSSet < Element, ElementOps, Base >;
  public:
    Cursor (IWBagOnKSSet < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}

    Boolean setToFirst ();
    Boolean setToNext  ();

    Element const& element ()
    { return Base::Cursor::element ().elem; }
    Boolean operator== (Cursor const& cursor)
    { return Base::Cursor::operator== (cursor) &&
             occurrence == cursor.occurrence; }
    Boolean operator!= (Cursor const& cursor)
    { return ! operator== (cursor); }
  };

                  IWBagOnKSSet             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IWBagOnKSSet             (IWBagOnKSSet < Element, ElementOps, Base > const&);

   IWBagOnKSSet < Element, ElementOps, Base >&    operator =                  (IWBagOnKSSet < Element, ElementOps, Base > const&);

                 ~IWBagOnKSSet             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWBagOnKSSet < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Boolean        contains                    (Element const&) const;

   Boolean        containsAllFrom             (IWBagOnKSSet < Element, ElementOps, Base > const&) const;

   Boolean        locate                      (Element const&, ICursor&)
                                               const;

   Boolean        locateOrAdd                 (Element const&);

   Boolean        locateOrAdd                 (Element const&,
                                               ICursor&);

   Boolean        remove                      (Element const&);

   INumber        numberOfOccurrences         (Element const&) const;

   Boolean        locateNext                  (Element const&, ICursor&)
                                               const;

   INumber        removeAllOccurrences        (Element const&);

   INumber        numberOfDifferentElements   () const;

   Boolean        setToNextDifferentElement   (ICursor&) const;

   Boolean        operator ==                 (IWBagOnKSSet < Element, ElementOps, Base > const&) const;

   Boolean        operator !=                 (IWBagOnKSSet < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           differenceWith              (IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           addUnion                    (IWBagOnKSSet < Element, ElementOps, Base > const&,
                                               IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           addIntersection             (IWBagOnKSSet < Element, ElementOps, Base > const&,
                                               IWBagOnKSSet < Element, ElementOps, Base > const&);

   void           addDifference               (IWBagOnKSSet < Element, ElementOps, Base > const&,
                                               IWBagOnKSSet < Element, ElementOps, Base > const&);

protected:

  void add (Element const&, INumber);

  struct RemoveEnv {
    Boolean (*predicate) (Element const&, void*);
    void* env;
    INumber removeCount;
  };
  static Boolean removePredicate (IBKSElm < Element > const&, void*);
  static Boolean cleanupPredicate (IBKSElm < Element > const&, void*);

  static ElementOps cvElementOps;

};

template < class Element, class ElementOps >
class IBKSSOps : public ElementOps {
public:
  void           assign(IBKSElm < Element >& e1,
			IBKSElm < Element > const& e2) const
		 { ElementOps::assign (e1.elem, e2.elem);
		   e1.occurrences = e2.occurrences; }
  Element const& key (IBKSElm < Element > const& e) const
                 { return e.elem; }
  ElementOps keyOps;
};

#define IDefineGBagOnGKeySortedSet(GKSSet, GBag) \
template < class Element, class ElementOps > \
class GKSSet ## ForGBag : \
  public GKSSet < IBKSElm < Element >, Element, \
		  IBKSSOps < Element, ElementOps > > { \
public: \
  GKSSet ## ForGBag (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    GKSSet < IBKSElm < Element >, Element, \
             IBKSSOps < Element, ElementOps > > (n, b) {} \
}; \
template < class Element, class ElementOps > \
class GBag : \
  public IWBagOnKSSet < Element, ElementOps, \
		         GKSSet ## ForGBag < Element, ElementOps > > { \
public: \
  GBag (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    IWBagOnKSSet < Element, ElementOps, \
		    GKSSet ## ForGBag < Element, ElementOps > > (n, b) {} \
};

#include <ibagkss.if>

#ifdef __IBMCPP__
#include <ibagkss.c>
#endif

#endif
