#ifndef _ICNRHDR_
#define _ICNRHDR_
/*******************************************************************************
* FILE NAME: icnrhdr.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     ICnrHandler                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   (C) Copyright IBM Corporation 1992                                         *
*   All Rights Reserved                                                        *
*   Licensed Materials * Property of IBM                                       *
*                                                                              *
* HISTORY:                                                                     *
* $Log:   R:/ibmclass/icnr/vcs/icnrhdr.hpv  $
//
//   Rev 1.4   26 Oct 1992 00:50:36   BOBLOVE
//External Beta Release
*                                                                              *
*******************************************************************************/
#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif

#ifndef _ICNRBEVT_
  #include <icnrbevt.hpp>
#endif


class ICnrHandler : public IHandler
{
/*******************************************************************************
* This class provides miscellaneous handler capability in the container.       *
*******************************************************************************/
public:

/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There is only one way to construct instances of this class:                  |
|   1. The default with no parameters.                                         |
------------------------------------------------------------------------------*/
  ICnrHandler();

/*------------------------ EVENT PROCESSING FUNCTIONS --------------------------
| These functions must be supplied by a derived class in order to provide      |
| processing for an edit event.                                                |
|   deltaReached   - The container has scrolled to the predefined delta.       |
|   treeCollapsed  - An object is being collapsed in the container.            |
|   treeExpanded   - An object is being expanded in the container.             |
------------------------------------------------------------------------------*/
virtual Boolean 
  deltaReached(ICnrQueryDeltaEvent& evt),
  treeCollapsed(IContainerObject* CollapsedObject),
  treeExpanded(IContainerObject* ExpandedObject);


protected:
/*----------------------------- IMPLEMENTATION ---------------------------------
| These functions are used to implement the class:                             |
| dispatchHandlerEvent - This function is responsible for routing the edit     |
|                        events to the appropriate event functions.            |
------------------------------------------------------------------------------*/
virtual Boolean 
  dispatchHandlerEvent(IEvent& evt);
 
};

/* INLINES */
inline Boolean ICnrHandler :: deltaReached(ICnrQueryDeltaEvent& evt)
       { return false; }
inline Boolean ICnrHandler :: treeCollapsed(IContainerObject* CollapsedObject)  
       { return false; }
inline Boolean ICnrHandler :: treeExpanded(IContainerObject* ExpandedObject)  
       { return false; }

#endif
