#ifndef _ICTLEVT_
  #define _ICTLEVT_
/*******************************************************************************
* FILE NAME: ictlevt.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IControlEvent - encapsulation of WM_CONTROL event                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   (C) Copyright IBM Corporation 1992                                         *
*   All Rights Reserved                                                        *
*   Licensed Materials = Property of IBM                                       *
*                                                                              *
*$Log:   R:/IBMCLASS/IBASEAPP/VCS/ICTLEVT.HPV  $                                                                         *
// 
//    Rev 1.23   25 Oct 1992 16:56:56   nunn
// changed library name to ICLUI
// 
//    Rev 1.22   25 Oct 1992 14:54:08   tsuji
// Format/documentation changes to match standard file header.
*******************************************************************************/
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

// Forward declarations for other classes:
class IControlEvent;  /* ctlevt */

class IControlEvent : public IEvent {
/*******************************************************************************
* This class encapsulates a WM_CONTROL message.  It provides two member        *
* functions in addition to what is provided by IEvent -- controlWindow() which *
* returns a pointer to the control that the WM_CONTROL message was generated   *
* on behalf of, and controlId() which returns the window id of that control.   *
* The window() and handle() functions will return information for the owner    *
* of the control.
*                                                                              *
* An IControlEvent object will be created by control event handlers, and       *
* passed to the virtual event processing function of the handler.  This        *
* callback function is intended to be overloaded by the user in order to       *
* process the WM_CONTROL event.                                                *
*******************************************************************************/
typedef IEvent Inherited;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There is one way to construct instances of this class:                       |
|   1. construct from an IEvent object                                         |
------------------------------------------------------------------------------*/
  IControlEvent ( IEvent& evt );

/*-------------------------------- ACCESSORS -----------------------------------
| These functions provide means of getting and setting the accessible          |
| attributes of instances of this class:                                       |
|   controlWindow - returns a pointer to the affected control                  |
|   controlId     - returns the window id of affected control                  |
------------------------------------------------------------------------------*/
IWindow* const
  controlWindow () const;
unsigned long
  controlId () const;
};


inline
IControlEvent::IControlEvent ( IEvent& evt )
/******************************************************************/
/* Constructor.                                                   */
/******************************************************************/
  : IEvent(evt)
{;}

#endif /* ICTLEVT */
