/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _IESSEQ_H
#define _IESSEQ_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class ElementOps, class Base >
class IWEqSeqOnSeq {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  public:
    Cursor (IWEqSeqOnSeq
	      < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWEqSeqOnSeq             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IWEqSeqOnSeq             (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   IWEqSeqOnSeq < Element, ElementOps, Base >&    operator =                  (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

                 ~IWEqSeqOnSeq             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Boolean        contains                    (Element const&) const;

   Boolean        containsAllFrom             (IWEqSeqOnSeq < Element, ElementOps, Base > const&) const;

   Boolean        locate                      (Element const&, ICursor&)
                                               const;

   Boolean        locateOrAdd                 (Element const&);

   Boolean        locateOrAdd                 (Element const&,
                                               ICursor&);

   Boolean        remove                      (Element const&);

   INumber        numberOfOccurrences         (Element const&) const;

   Boolean        locateNext                  (Element const&, ICursor&)
                                               const;

   INumber        removeAllOccurrences        (Element const&);

   Boolean        operator ==                 (IWEqSeqOnSeq < Element, ElementOps, Base > const&) const;

   Boolean        operator !=                 (IWEqSeqOnSeq < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   void           differenceWith              (IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   void           addUnion                    (IWEqSeqOnSeq < Element, ElementOps, Base > const&,
                                               IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   void           addIntersection             (IWEqSeqOnSeq < Element, ElementOps, Base > const&,
                                               IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   void           addDifference               (IWEqSeqOnSeq < Element, ElementOps, Base > const&,
                                               IWEqSeqOnSeq < Element, ElementOps, Base > const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   Boolean        setToLast                   (ICursor&) const;

   Boolean        setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   Boolean        isFirst                     (ICursor const&) const;

   Boolean        isLast                      (ICursor const&) const;

   long           compare                     (IWEqSeqOnSeq < Element, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

   void           addAsFirst                  (Element const&);

   void           addAsFirst                  (Element const&,
                                               ICursor&);

   void           addAsLast                   (Element const&);

   void           addAsLast                   (Element const&,
                                               ICursor&);

   void           addAsNext                   (Element const&,
                                               ICursor&);

   void           addAsPrevious               (Element const&,
                                               ICursor&);

   void           addAtPosition               (IPosition,
                                               Element const&);

   void           addAtPosition               (IPosition,
                                               Element const&,
                                               ICursor&);

   void           sort                        (long (*comparisonFunction)
                                               (Element const&,
                                               Element const&));

   Boolean        locateFirst                 (Element const&, ICursor&)
                                               const;

   Boolean        locateLast                  (Element const&,
                                               ICursor&) const;

   Boolean        locatePrevious              (Element const&,
                                               ICursor&) const;

protected:

  static Boolean  isContained    (Element const&, void* env);
  static Boolean  isNotContained (Element const&, void* env);
  static Boolean  isEqual        (Element const&, void* env);

  static ElementOps cvElementOps;

};

#define IDefineGEqualitySequenceOnGSequence(GSeq, GEqSeq) \
template < class Element, class ElementOps > \
class GEqSeq : \
  public IWEqSeqOnSeq < Element, ElementOps, \
		        GSeq < Element, ElementOps > > { \
public: \
  GEqSeq (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    IWEqSeqOnSeq < Element, ElementOps, \
		   GSeq < Element, ElementOps > > (n, b) {} \
};

#include <iesseq.if>

#ifdef __IBMCPP__
#include <iesseq.c>
#endif

#endif
