#ifndef _IGROUPBX_
  #define _IGROUPBX_
/*******************************************************************************
* FILE NAME: Igroupbx.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*    Igroupbx - This class creates and manages the group box control window.   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
/* $Log:   R:/IBMCLASS/IBASECTL/VCS/IGROUPBX.HPV  $
//
//   Rev 1.6   25 Oct 1992 17:05:26   nunn
//changed library name to ICLUI
//
//   Rev 1.5   25 Oct 1992 10:54:30   boezeman
//Add documentation and converted file to skeleton.hpp format.

      Rev 1.4   13 Oct 1992 21:22:12   tsuji
   Correct comments around log directive.

      Rev 1.3   13 Oct 1992 20:56:54   tsuji
   Correct log directive, remove IBM Confidential classification.

      Rev 1.2   13 Oct 1992 19:05:50   tsuji
   Remove no-op virtual destructor, add Inherited typedef.
*******************************************************************************/
#ifndef _ITEXTCTL_
  #include <itextctl.hpp>
#endif

// Forward declarations for other classes:
class IGroupBox;  /* gbx */
class IRectangle;
class ISize;
class IWindowHandle;
#ifndef _IBITFLAG_
  #include <ibitflag.hpp>
#endif

class IGroupBox : public ITextControl  {
/*******************************************************************************
* This class creates and manages the group box control window.                 *
*                                                                              (
* To use this class, create an instance of this class as follows:              *
*   IGroupBox gbxGroup(ID_GROUPBOX, this, IRectangle(10,10,110,210));          *
*   gbxGroup.setText("Colors");                                                *
* EXAMPLE:                                                                     *
*   <sample code>                                                              *
*******************************************************************************/
typedef ITextControl
  Inherited;

public:
INESTEDBITFLAGCLASSDEF1(Style, IGroupBox, IWindow);
                                  // style class definition
static const Style
  autosize;
static const Style&
  defStyle;
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 3 ways to construct instances of this class:                       |
|   1. default                                                                 |
|   2. From a Dialog Template                                                  |
|   3. From a Window                                                           |
------------------------------------------------------------------------------*/
  IGroupBox(unsigned long Id,
            const IWindow* parent,
            const IWindow* owner,
            const IRectangle& initial,
            Style style = defStyle);

  IGroupBox(unsigned long Id,
            const IWindow* parentDialog);

  IGroupBox(IWindowHandle handle);

/*-------------------------------- STYLES --------------------------------------
| These function provide means of getting and setting the default style        |
| attributes of instances of this class:                                       |
|   defaultStyle    - returns the group box control default style setting      |
|   setDefaultStyle - sets the group box control default style                 |
------------------------------------------------------------------------------*/
static Style
  defaultStyle();
static void
  setDefaultStyle(Style style);

protected:
/*----------------------------- LAYOUT SIZE ------------------------------------
| calcMinSize - returns the minimum size that this group box control should be.|
------------------------------------------------------------------------------*/
virtual ISize
  calcMinSize();

static Style
  classDefaultStyle;

private:
/*--------------------------------- PRIVATE ----------------------------------*/
  IGroupBox(const IGroupBox&);
//IGroupBox& operator=(const IGroupBox&);

static const Style
  groupBox;
};  // class IGroupBox

INESTEDBITFLAGCLASSFUNCS(Style, IGroupBox);
                                  // global style functions
#endif  /* _IGROUPBX_ */
