#ifndef _IHELP_
#define _IHELP_
/*******************************************************************************
* FILE NAME: ihelp.hpp.                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IHelp - This GUI help instance class                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*$Log:   R:/IBMCLASS/IBASEAPP/VCS/IHELP.HPV  $
//
//   Rev 1.1   26 Oct 1992 10:48:58   nunn
//Kevin's changes for 10/26
   
      Rev 1.1   26 Oct 1992 00:00:01   kleong
   Miscellaneous improvements.
*******************************************************************************/
#ifndef _IBASE_
  #include <ibase.hpp>
#endif
#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif

// Forward declarations for other classes:
class IWindow;
class IWindowHandle;
class ISize;
class IResourceId;

class IHelp : public IBase {
/*******************************************************************************
* The IHelp class defines help panels using the system help Information        *
* Presentation Facility                                                        *
*                                                                              *
* <2 to 3 paragraphs describing how objects of this class are used>            *
*                                                                              *
* EXAMPLE:                                                                     *
*   <sample code>                                                              *
*******************************************************************************/
typedef IBase Inherited;
public :
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are one way to construct instances of this class:                      |
|   1. Construct a help instance given the help table resource ID,
       the active application window and the help file name.
------------------------------------------------------------------------------*/

  IHelp(const IResourceId& resid, const IWindow* activeWindow,
        const char* helpFileName);
  ~IHelp();

/*-------------------------------- ACCESSORS -----------------------------------
| These function provide means of getting and setting the accessible           |
| attributes of instances of this class:  
    associate           - Associates the help instance with the
                          application window chain
    handle              - Returns the help instance window handle
    setActiveWindow     - Sets the active window to receive help
                          window events
    setCoverpageSize    - Sets the size of the help window coverpage
    setHelpFile         - Sets a list of help library file
    setHelpForHelp      - Replaces the Help For Help panel
    setHelpWindowAccel  - Replaces the help window accelerator table
    setTitle            - Sets the help window title
    setUsingHelp        - Replaces the Using Help panel
    setHelpWindowMenu   - Replaces the help window menu
    setHelpTable        - Replaces the help table resource
------------------------------------------------------------------------------*/
const IWindowHandle
  &handle() const;

IHelp 
  &associate(const IWindow* windowChain),
  &setActiveWindow(const IWindow* window),
  &setCoverpageSize(const ISize& newSize),
  &setHelpFile(const char* helpFile),
  &setHelpForHelp(unsigned long helpId),
  &setHelpWindowAccel(const IResourceId& accelResId),
  &setTitle(const char* title),
  &setTitle(const IResourceId& id),
  &setUsingHelp(unsigned long helpId);

//     IHelp& setHelpWindowMenu(const IResourceId& menuResId);
//     IHelp& setHelpTable(const IResourceId& helpResId);

enum Indicator {visible, invisible, flash};
IHelp
  &setPanelId(Indicator ind);

/*-------------------------------- ACCESSORS -----------------------------------
| These function provide means of showing the help window panels.              |
    show  -  Displays specific help window panel
    hide  -  Dismisses the help window
------------------------------------------------------------------------------*/
enum HelpType {index, general, keys, contents};
IHelp
  &show(HelpType type),
  &show(unsigned long id),
  &show(const char* panelName),
  &hide(); 

/*--------------------------------- PRIVATE ----------------------------------*/
private :
void
  *phelpdata;
IWindowHandle
  helpHandle;
};
#endif
