#ifndef _IHELPHDR_
#define _IHELPHDR_
/*******************************************************************************
* FILE NAME: ihelphdr.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IHelpHandler - This class deals with help events                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*$Log:   R:/IBMCLASS/IBASEAPP/VCS/IHELPHDR.HPV  $
//
//   Rev 1.1   26 Oct 1992 10:49:08   nunn
//Kevin's changes for 10/26
   
      Rev 1.1   26 Oct 1992 00:00:01   kleong
   Miscellaneous improvements.
*******************************************************************************/
#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif

// Forward declarations for other classes:
class IHelpHandler;
class ICommandEvent;

class IHelpHandler : public IHandler {
/*******************************************************************************
* The IHelpHandler class deals with help window events.                        *
*                                                                              *
* EXAMPLE:                                                                     *
*   <sample code>                                                              *
*******************************************************************************/
typedef IHandler Inherited;
protected:

/*---------------------------- EVENT HANDLING ----------------------------------
| These function provide means of handling help events                         |
    helpCommand     - Overrides this function to perform process
                      when a tailored menu bar item is selected
    hypertextSelect - Overrides this function to provide hypertext
                      processing
    controlSelect   - Overrides this function to perform action
                      when help window control is selected
    notifyOpen      - Overrides this function to perform action
                      when component of help window is opened
    showTutorial    - Overrides this function to start the
                      application tutorial
    showKeysHelp    - Overrides this function to display Keys Help
                      panel
------------------------------------------------------------------------------*/
virtual Boolean
  helpCommand(ICommandEvent& cmdevt) {return false;}
virtual Boolean
  hypertextSelect(unsigned long linkId) {return false;}

static const unsigned long
  previousButtonId,
  searchButtonId,
  printButtonId,
  indexButtonId,
  contentsButtonId,
  backButtonId,
  forwardButtonId,
  tutorialButtonId;

virtual Boolean 
  controlSelect(unsigned long controlResId) {return false;}

enum Notification 
  { openCoverpage, openPage, swapPage, openIndex, openContents,
    openHistory, openLibrary, openSearchHitList};

virtual Boolean 
  notifyOpen(Notification n) {return false;}
        //setResult(true) - IPF will not format the controls
        //     or resize window otherwise process as normal

virtual Boolean 
  showTutorial() {return false;}
virtual Boolean 
  showKeysHelp() {return false;}
        // setResult keyHelpId

/*-------------------------------- OVERRIDES -----------------------------------
| This class overrides the following inherited functions:                      |
|   dispatchHandlerEvent -  Processes help error messages and dispatch
                            help event handling functions
------------------------------------------------------------------------------*/
virtual Boolean 
  dispatchHandlerEvent(IEvent& evt);
};
#endif /* IHELPHDR */
