/* Copyright (c) IBM Corp. 1992 */
#ifndef _IIAVLKSS_H
#define _IIAVLKSS_H

#include <iglobals.h>

class IAvlKeySortedSetImpl {
public:

  class Node {
    friend class IAvlKeySortedSetImpl;
    Node*          ivLeft;
    Node*          ivRight;
    Node*          ivParent;
    short          ivBalanced;
	
    Node*&         link (short);
  public:
    Node           ();
  };

  class Operations {
    virtual Node*    newNode              (void const* element) const = 0;
    virtual Node*    copyNode             (void const* node) const = 0;
    virtual void     deleteNode           (void* node) const = 0;
    virtual void*    elementAt            (void* node) const = 0;
    virtual void const*
		     constElementAt       (void const* node) const = 0;
    virtual void const*
		     keyAt                (void const* node) const = 0;
    virtual long     compareKeyTo         (void const* node1,
					   void const* node2) const = 0;
    virtual long     compareKeyToElement  (void const* node,
					   void const* element) const = 0;
    virtual long     compareKeyToKey      (void const* node,
					   void const* key) const = 0;
    virtual void     copyFrom             (void* node,
					   void const* element) const = 0;
    virtual Boolean  constantFunctionIteration
                                          (void *function,
                                           void* env,
                                           void const* node) = 0;
    virtual Boolean  functionIteration    (void *function,
                                           void* env,
                                           void* node) = 0;
    virtual Boolean  constantIteratorIteration
				          (void* iterator,
                                           void const* node) = 0;
    virtual Boolean  iteratorIteration    (void* iterator,
                                           void* node) = 0;

    virtual long     functionComparison   (void *compareFunction,
                                           void const* node1,
                                           void const* node2) = 0;
    friend class IAvlKeySortedSetImpl;
  };

private :
   Operations*         ivOps;
   Node*               ivRoot;
   INumber             ivNumberOfElements;

   void const*         keyAt               (Node const*) const;
   void                deleteTree          (Node*);
   void                balanceL            (Node*&,
                                            Boolean&);
   void                balanceR            (Node*&,
                                            Boolean&);
   Boolean             removeBalanced      (Node*);
   Boolean             removeBalanced      (Node*,
                                            Node*&,
                                            Boolean&);
   Boolean             searchAddPosition   (void const* element,
                                            Node*&,
                                            Node*&,
                                            Node*&,
                                            Node*&);
   void                insertAtPosition    (Node*,
                                            Node*,
                                            Node*,
                                            Node*);
   void                replaceNode         (Node*,
                                            Node*);
   void                disconnect          (Node*&,
                                            Boolean&);
   void                disconnectAndRemove (Node*&,
                                            Boolean& h);
   Node*               copySubtree         (Node const*);
   Boolean             locateElementWithKeyOfElement
                                           (void const* element,
                                            Node*&) const;
   Boolean             containsAllKeysFromSubtree
                                           (Node const*) const;
   void                unionWithSubtree    (Node const*);

   Boolean             allElementsDoSubtree(void* function,
                                            void* environment,
                                            Node const*) const;

   Boolean             allElementsDoSubtree(void* function,
                                            void* environment,
                                            Node*);

   Boolean             allElementsDoSubtree(void* iterator,
                                            Node const*) const;

   Boolean             allElementsDoSubtree(void* iterator,
                                            Node*);

public:
                       IAvlKeySortedSetImpl              (Operations*);
                       IAvlKeySortedSetImpl              (IAvlKeySortedSetImpl const&,
                                                  Operations*);
   void                operator=           (IAvlKeySortedSetImpl const&);

  virtual             ~IAvlKeySortedSetImpl              ();

   Boolean             add                 (void const* element);
   Boolean             add                 (void const* element,
                                            Node*&);
   void                addAllFrom          (IAvlKeySortedSetImpl const&);
   void const*         elementAt           (Node const*) const;
   void*               elementAt           (Node*);
   void                removeAt            (Node*);
   INumber             removeAll           (void* predicate,
                                            void* environment);
   void                replaceAt           (Node*,
                                            void const* element);
   void                removeAll           ();
   Boolean             isBounded           () const;
   IPosition           numberOfElements    () const;
   Boolean             isEmpty             () const;
   Boolean             isFull              () const;

   Boolean             setToFirst          (Node*&) const;
   Boolean             setToNext           (Node*&) const;

   Boolean             allElementsDo       (void* function,
                                            void* environment) const;

   Boolean             allElementsDo       (void* function,
                                            void* environment);

   Boolean             allElementsDo       (void* iterator) const;

   Boolean             allElementsDo       (void* iterator);

   INumber             removeAllOccurences (void const* element);

   Boolean             containsAllKeysFrom (IAvlKeySortedSetImpl const&) const;
   Boolean             containsElementWithKey
                                           (void const* key) const;
   Boolean             locateElementWithKey(void const* key,
                                            Node*&) const;
   Boolean             replaceElementWithKey
                                           (void const* element,
                                            Node*&);
   Boolean             replaceElementWithKey
                                           (void const* element);
   Boolean             addOrReplaceElementWithKey
                                           (void const* element,
                                            Node*&);
   Boolean             addOrReplaceElementWithKey
                                           (void const* element);
   Boolean             locateOrAddElementWithKey
                                           (void const* element,
                                            Node*&);
   Boolean             locateOrAddElementWithKey
                                           (void const* element);
   Boolean             removeElementWithKey(void const* key);
   void const*         elementWithKey      (void const* key) const;
   void*               elementWithKey      (void const* key);
   INumber             numberOfElementsWithKey
                                           (void const* key) const;

   void                removeFirst         ();
   void                removeLast          ();
   void                removeAtPosition    (IPosition);
   void const*         firstElement        () const;
   void const*         lastElement         () const;
   void const*         elementAtPosition   (IPosition) const;
   Boolean             setToLast           (Node*&) const;
   Boolean             setToPrevious       (Node*&) const;
   void                setToPosition       (IPosition,
                                            Node*&) const;
   Boolean             isFirst             (Node const*) const;
   Boolean             isLast              (Node const*) const;

   long                compare             (IAvlKeySortedSetImpl const&,
                                            void *function) const;

   Boolean             checkNode           (Node const*, Node*) const;
   Boolean             checkNode           (Node const*) const;
};

#endif
