/* Copyright (c) IBM Corp. 1992 */
#ifndef _IILNSEQ_H
#define _IILNSEQ_H

#include <iglobals.h>
#include <icursor.h>

class ILinkedSequenceImpl
{
public:
  class Node
  {
  protected:
    friend class ILinkedSequenceImpl;

    Node* ivNext;
    Node* ivPrevious;
  };

  class Operations
  {
    virtual Node*       newNode              (void const* node) const = 0;
    virtual void        deleteNode           (void* node) const = 0;

    virtual Boolean     constantFunctionIteration
                                             (void *function,
                                              void* env,
                                              void const* node) = 0;
    virtual Boolean     functionIteration    (void *function,
                                              void* env,
                                              void* node) = 0;
    virtual Boolean     constantIteratorIteration
				             (void* iterator,
                                              void const* node) = 0;
    virtual Boolean     iteratorIteration    (void* iterator,
                                              void* node) = 0;

    virtual long        functionComparison   (void *compareFunction,
                                              void const* node1,
                                              void const* node2) = 0;

    friend class ILinkedSequenceImpl;
  };

private :

  Node *ivFirst;
  Node *ivLast;
  INumber ivNumberOfElements;
  Operations *ivOps;


public :

                  ILinkedSequenceImpl         (Operations*);

                  ILinkedSequenceImpl         (ILinkedSequenceImpl const&,
					       Operations*);

                 ~ILinkedSequenceImpl         ();

   void           addAllFrom                  (ILinkedSequenceImpl const&);

   void           removeAt                    (Node*);

   INumber        removeAll                   (void* predicate,
                                               void* environment);

   void           removeAll                   ();

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   Boolean        setToFirst                  (Node*&) const;

   Boolean        setToNext                   (Node*&) const;

   Boolean        allElementsDo               (void* function, void* env);

   Boolean        allElementsDo               (void* function, void* env) const;

   Boolean        allElementsDo               (void* iterator);

   Boolean        allElementsDo               (void* iterator) const;

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Boolean        setToLast                   (Node*&) const;

   Boolean        setToPrevious               (Node*&) const;

   void           setToPosition               (IPosition, Node*&) const;

   Boolean        isFirst                     (Node const*) const;

   Boolean        isLast                      (Node const*) const;

   long           compare                     (ILinkedSequenceImpl const&,
                                               void* comparisonFunction) const;

   Node*          addAsFirst                  (Node*);

   Node*          addAsLast                   (Node*);

   Node*          addAsNext                   (Node*, Node*);

   Node*          addAsPrevious               (Node*, Node*);

   Node*          addAtPosition               (IPosition, Node*);

   void           sort                        (void* comparisonFunction);

   Boolean        checkNode                   (Node const*) const;

};

#endif
