/* Copyright (c) IBM Corp. 1992 */
#ifndef _IITBTRE_H
#define _IITBTRE_H

#include <iglobals.h>

class ITabularTreeImpl
{
public:
  class Node
  {
    friend class ITabularTreeImpl;
  public:
    Node** ivParent;
    Node* ivChildren [1];
    // ivChildren contains in fact numberOfChildren node pointers,
    // where numberOfChildren is the template argument of the
    // corresponding ITabularTree.
  };

  Node* ivRoot;       
  INumber ivNumberOfChildren;

  ITabularTreeImpl () : ivRoot (0)
  {}

  void    initNode                   (Node* node, Node** parent) const;
  INumber numberOfSubtreeElements    (Node const*) const;
  INumber numberOfSubtreeLeaves      (Node const*) const;
  Boolean isLeaf                     (Node const*) const;
  INumber position                   (Node const*) const;

  Boolean setToParent                (Node**& node) const;
  Boolean setToFirstExistingChild    (Node**& node) const;
  Boolean setToNextExistingChild     (Node**& node) const;
  Boolean setToLastExistingChild     (Node**& node) const;
  Boolean setToPreviousExistingChild (Node**& node) const;

  Boolean setToFirst                 (Node**& node,
				      ITreeIterationOrder order) const;
  Boolean setToNext                  (Node**& node,
				      ITreeIterationOrder order) const;
  Boolean setToLast                  (Node**& node,
				      ITreeIterationOrder order) const;
  Boolean setToPrevious              (Node**& node,
				      ITreeIterationOrder order) const;

  Boolean allElementsDo              (void* function,
				      ITreeIterationOrder order,
				      void* env,
                                      Boolean (*apply) (void *function,
                                                        void* env,
                                                        void* node),
				      Node* subtreeRoot);

  Boolean allElementsDo              (void* function,
				      ITreeIterationOrder order,
				      void* env,
				      Boolean (*apply) (void *function,
						        void* env,
						        void const* node),
				      Node* subtreeRoot) const;

  Boolean allElementsDo              (void* iterator,
				      ITreeIterationOrder order,
				      Boolean (*apply) (void* iterator,
						        void* node),
				      Node* subtreeRoot);

  Boolean allElementsDo              (void* iterator,
				      ITreeIterationOrder order,
				      Boolean (*apply) (void* iterator,
							void const* node),
				      Node* subtreeRoot) const;

};

#endif
