#ifndef _IMCEVT_
#define _IMCEVT_
/*******************************************************************************
* FILE NAME: imcevt.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IMouseClickEvent - the mouse click event information                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*$Log:   R:/IBMCLASS/IBASEAPP/VCS/IMCEVT.HPV  $                                                                         *
// 
//    Rev 1.22   25 Oct 1992 18:00:28   nunn
// Brad Broyles changes for external beta

      Rev 1.1   01 Jan 1989 12:00:00   LM & KKL
   Initiate code
      Rev 1.2   01 May 1991 12:00:00   KKL
   Combine all events into 1 file - modified for better performance
      Rev 1.3   01 Oct 1991 12:00:00   KKL
   Update per R2.0 spec.
      Rev 1.4   09 Aug 1992 12:00:00   KKL
   Mod. for handlers
      Rev 1.5   21 Oct 1992 17:00:00   WBB
   Amended to match skeleton.hpp
*******************************************************************************/

// Forward declarations for other classes:
class IMouseClickEvent;
class IWindowHandle;
class IPoint;
#if !defined( _IEVENT_ ) && !defined( I_NO_RELATED_HPP )
  #include <ievent.hpp>
#endif

class IMouseClickEvent : public IEvent {
/*******************************************************************************
* The IMouseClickEvent class provides mouse click event information.  A mouse  *
* click event event is caused by the user's pressing the mouse button down,    *
* up, click, and double-click.                                                 *
*******************************************************************************/
typedef IEvent Inherited;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There is one way to construct instances of this class:                       |
|   1. from an instance of the IEvent class                                    |
------------------------------------------------------------------------------*/
  IMouseClickEvent(IEvent& evt);

/*-------------------------------- FUNCTIONS -----------------------------------
| mousePosition - return the mouse pointer position                            |
| handleOnTop - return the handle of the window the mouse is over              |
| mouseNumber - return which mouse button was operated on                      |
| mouseAction - return the action taken on the mouse                           |
------------------------------------------------------------------------------*/
IPoint
  mousePosition() const;
IWindowHandle
  handleOnTop() const;

  enum MouseNumber {button1, button2, button3, button12};
MouseNumber
  mouseNumber() const;

  enum MouseAction {click, doubleClick, down, up};
MouseAction
  mouseAction() const;
}; // IMouseClickEvent

/*--------------------------------- INLINES ----------------------------------*/
#ifndef I_NO_INLINES
  #include <imcevt.inl>
#endif

#endif /* _IMCEVT_ */
