/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _IPQUKSB_H
#define _IPQUKSB_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class Key, class ElementOps, class Base >
class IWPQueOnKSBag {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  public:
    Cursor (IWPQueOnKSBag < Element, Key, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWPQueOnKSBag             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IWPQueOnKSBag             (IWPQueOnKSBag < Element, Key, ElementOps, Base > const&);

   IWPQueOnKSBag < Element, Key, ElementOps, Base >&    operator =                  (IWPQueOnKSBag < Element, Key, ElementOps, Base > const&);

                 ~IWPQueOnKSBag             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWPQueOnKSBag < Element, Key, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Key const&     key                         (Element const&) const;

   Boolean        containsElementWithKey      (Key const&) const;

   Boolean        containsAllKeysFrom         (IWPQueOnKSBag < Element, Key, ElementOps, Base > const&) const;

   Boolean        locateElementWithKey        (Key const&, ICursor&)
                                               const;

   Boolean        replaceElementWithKey       (Element const&);

   Boolean        replaceElementWithKey       (Element const&,
                                               ICursor&);

   Boolean        locateOrAddElementWithKey   (Element const&);

   Boolean        locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   Boolean        addOrReplaceElementWithKey  (Element const&);

   Boolean        addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   Boolean        removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

   INumber        numberOfElementsWithKey     (Key const&) const;

   Boolean        locateNextElementWithKey    (Key const&,
                                               ICursor&) const;

   INumber        removeAllElementsWithKey    (Key const&);

   INumber        numberOfDifferentKeys       () const;

   Boolean        setToNextWithDifferentKey   (ICursor&) const;

   void           removeFirst                 ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   Boolean        setToLast                   (ICursor&) const;

   Boolean        setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   Boolean        isFirst                     (ICursor const&) const;

   Boolean        isLast                      (ICursor const&) const;

   long           compare                     (IWPQueOnKSBag < Element, Key, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

   void           enqueue                     (Element const&);

   void           enqueue                     (Element const&,
                                               ICursor&);

   void           dequeue                     ();

   void           dequeue                     (Element&);

};

#define IDefineGPriorityQueueOnGKeySortedBag(GKSBag, GPrioQue) \
template < class Element, class Key, class ElementOps > \
class GPrioQue : \
  public IWPQueOnKSBag < Element, Key, \
			    ElementOps, GKSBag < Element, Key, ElementOps > > { \
public: \
  GPrioQue (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    IWPQueOnKSBag < Element, Key, ElementOps, \
		       GKSBag < Element, Key, ElementOps > > (n, b) {} \
};

#include <ipquksb.if>

#endif
