/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _IRKSSET_H
#define _IRKSSET_H

#include <iaksset.h>

template < class Element, class Key, class Base >
class IRKeySortedSet : public virtual IAKeySortedSet < Element, Key > {
  Base &ivBase;
public:

       IRKeySortedSet (Base& base) : ivBase (base) {};

                 ~IRKeySortedSet             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Key const&     key                         (Element const&) const;

   Boolean        containsElementWithKey      (Key const&) const;

   Boolean        locateElementWithKey        (Key const&, ICursor&)
                                               const;

   Boolean        replaceElementWithKey       (Element const&);

   Boolean        replaceElementWithKey       (Element const&,
                                               ICursor&);

   Boolean        locateOrAddElementWithKey   (Element const&);

   Boolean        locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   Boolean        addOrReplaceElementWithKey  (Element const&);

   Boolean        addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   Boolean        removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   Boolean        setToLast                   (ICursor&) const;

   Boolean        setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   Boolean        isFirst                     (ICursor const&) const;

   Boolean        isLast                      (ICursor const&) const;

};

#ifdef __IBMCPP__
#include <irksset.c>
#endif

#endif
