#ifndef _ISCRLEVT_
  #define _ISCRLEVT_
/*******************************************************************************
* FILE NAME: iscrlevt.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IScrollEvent - Represents a scroll event for an IScrollBar object.       *
*                                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*$Log:   S:/ibmclass/ibaseapp/vcs/iscrlevt.hpv  $                                                                         *
// 
//    Rev 1.3   26 Oct 1992 00:36:16   tsuji
// Format/document in the style of the standard header file.
*******************************************************************************/
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

// Forward declarations for other classes:
class IScrollEvent;
class IScrollBar;

class IScrollEvent : public IEvent {
/*******************************************************************************
* This class represents a scroll event for an IScrollBar object.  This event   *
* update its appearance on the screen.  This event object is created by an     *
* IScrollHandler object and passed to one of its virtual functions for         *
* processing.                                                                  *
*******************************************************************************/
typedef IEvent Inherited;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There is 1 way to construct instances of this class:                         |
|   1. construct from an IEvent                                                |
|      This is the constructor used by the IScrollHandler class.               |
------------------------------------------------------------------------------*/
  IScrollEvent ( IEvent& evt ) : Inherited( evt ) {;}

/*--------------------------------- ACCESSORS ----------------------------------
| These functions return information related to the scroll event.              |
|   newSliderPosition - returns the position of the slider (this is only       |
|                       valid for a scroll event passed to the                 |
|                       IScrollHandler::sliderTrack() function)                |
|   sliderMovedBy     - returns the number of units that the scroll has        |
|                       caused the slider to move (a negative value shows a    |
|                       backwards scroll)                                      |
|   scrollBarWindow   - returns a pointer to the IScrollBar object that the    |
|                       scroll event applies to                                |
------------------------------------------------------------------------------*/
long
  newSliderPosition ( ) const,
  sliderMovedBy     ( ) const;
IScrollBar* const
  scrollBarWindow   ( ) const;
};
#endif /* ISCRLEVT */
