#ifndef _ISETBUT_
  #define _ISETBUT_
/*******************************************************************************
* FILE NAME: isetbut.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     Isetbut - This is an abstract base class for the I3StateCheckBox,        *
*                ICheckBox, and IRadioButton control window classes.           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
* $Log:   R:/IBMCLASS/IBASECTL/VCS/ISETBUT.HPV  $
//
//   Rev 1.7   25 Oct 1992 17:06:06   nunn
//changed library name to ICLUI
//
//   Rev 1.6   25 Oct 1992 10:57:10   boezeman
//Add documentation and converted file to skeleton.hpp format.

      Rev 1.5   13 Oct 1992 21:22:36   tsuji
   Correct comments around log directive.

      Rev 1.4   13 Oct 1992 20:57:50   tsuji
   Correct log directive, remove IBM Confidential classification.

      Rev 1.3   13 Oct 1992 19:06:12   tsuji
   Remove no-op virtual destructor, add Inherited typedef.
*******************************************************************************/
#ifndef _IBUTTON_
  #include <ibutton.hpp>
#endif

// Forward declarations for other classes:
class ISettingButton;  /* setbut */
class IRectangle;
#ifndef _IBITFLAG_
  #include <ibitflag.hpp>
#endif

class ISettingButton : public IButton  {
/*******************************************************************************
* This is an abstract base class for the I3StateCheckBox, ICheckBox, and       *
* IRadioButton control window classes.                                         *
*                                                                              *
* This class should not be constructed directly.                               *
*                                                                              *
* EXAMPLE:                                                                     *
*   <sample code>                                                              *
*******************************************************************************/
typedef IButton
  Inherited;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There is 1 way to construct instances of this class:                         |
|   1. default                                                                 |
------------------------------------------------------------------------------*/
  ISettingButton() {;}

/*------------------------ TEST/SELECT BUTTON ----------------------------------
| isSelected   -  Return true if the button is selected.                       |
| select       -  Selects the button.                                          |
| unselect     -  Unselects the button.                                        |
------------------------------------------------------------------------------*/
Boolean
  isSelected() const;
void
 select(),
 unselect();

/*------------------------ TEST/SET AUTO SELECT STATE --------------------------
| enableAutoSelect  - virtual declaration for enabling the auto select style   |
|                     for a button.                                            |
| disableAutoSelect - virtual declaration for disabling the auto select style  |
|                     for a button.                                            |
| isAutoSelect      - virtual declaration for return whether the auto select   |
|                     style is set for a button                                |
------------------------------------------------------------------------------*/
virtual void
  enableAutoSelect() = 0,
  disableAutoSelect() = 0;
virtual Boolean
  isAutoSelect() const = 0;

private:
  ISettingButton(const ISettingButton&);
//ISettingButton& operator=(const ISettingButton&);
};  // class ISettingButton
#endif  /* _ISETBUT_ */
