/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _ISETKS_H
#define _ISETKS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class ElementOps, class Base >
class IWSetOnKeySet {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  public:
    Cursor (IWSetOnKeySet < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWSetOnKeySet             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IWSetOnKeySet             (IWSetOnKeySet < Element, ElementOps, Base > const&);

   IWSetOnKeySet < Element, ElementOps, Base >&    operator =                  (IWSetOnKeySet < Element, ElementOps, Base > const&);

                 ~IWSetOnKeySet             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWSetOnKeySet < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Boolean        contains                    (Element const&) const;

   Boolean        containsAllFrom             (IWSetOnKeySet < Element, ElementOps, Base > const&) const;

   Boolean        locate                      (Element const&, ICursor&)
                                               const;

   Boolean        locateOrAdd                 (Element const&);

   Boolean        locateOrAdd                 (Element const&,
                                               ICursor&);

   Boolean        remove                      (Element const&);

   Boolean        operator ==                 (IWSetOnKeySet < Element, ElementOps, Base > const&) const;

   Boolean        operator !=                 (IWSetOnKeySet < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWSetOnKeySet < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWSetOnKeySet < Element, ElementOps, Base > const&);

   void           differenceWith              (IWSetOnKeySet < Element, ElementOps, Base > const&);

   void           addUnion                    (IWSetOnKeySet < Element, ElementOps, Base > const&,
                                               IWSetOnKeySet < Element, ElementOps, Base > const&);

   void           addIntersection             (IWSetOnKeySet < Element, ElementOps, Base > const&,
                                               IWSetOnKeySet < Element, ElementOps, Base > const&);

   void           addDifference               (IWSetOnKeySet < Element, ElementOps, Base > const&,
                                               IWSetOnKeySet < Element, ElementOps, Base > const&);

protected:

  static Boolean  isContained    (Element const&, void* env);
  static Boolean  isNotContained (Element const&, void* env);

};

#define IDefineGSetOnGKeySet(GKeySet, GSet) \
template < class Element, class ElementOps > \
class GKeySet ## ForGSet : \
  public GKeySet < Element, Element, \
		   IOpsWithKey < Element, ElementOps > > { \
public: \
  GKeySet ## ForGSet (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    GKeySet < Element, Element, \
              IOpsWithKey < Element, ElementOps > > (n, b) {} \
}; \
template < class Element, class ElementOps > \
class GSet : \
  public IWSetOnKeySet < Element, ElementOps, \
		         GKeySet ## ForGSet < Element, ElementOps > > { \
public: \
  GSet (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    IWSetOnKeySet < Element, ElementOps, \
		    GKeySet ## ForGSet < Element, ElementOps > > (n, b) {} \
};

#include <isetks.if>

#ifdef __IBMCPP__
#include <isetks.c>
#endif

#endif
