/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _ISETKSS_H
#define _ISETKSS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class ElementOps, class Base >
class IWSetOnKSSet {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  public:
    Cursor (IWSetOnKSSet < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWSetOnKSSet             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IWSetOnKSSet             (IWSetOnKSSet < Element, ElementOps, Base > const&);

   IWSetOnKSSet < Element, ElementOps, Base >&    operator =                  (IWSetOnKSSet < Element, ElementOps, Base > const&);

                 ~IWSetOnKSSet             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWSetOnKSSet < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Boolean        contains                    (Element const&) const;

   Boolean        containsAllFrom             (IWSetOnKSSet < Element, ElementOps, Base > const&) const;

   Boolean        locate                      (Element const&, ICursor&)
                                               const;

   Boolean        locateOrAdd                 (Element const&);

   Boolean        locateOrAdd                 (Element const&,
                                               ICursor&);

   Boolean        remove                      (Element const&);

   Boolean        operator ==                 (IWSetOnKSSet < Element, ElementOps, Base > const&) const;

   Boolean        operator !=                 (IWSetOnKSSet < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           differenceWith              (IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           addUnion                    (IWSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           addIntersection             (IWSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSetOnKSSet < Element, ElementOps, Base > const&);

   void           addDifference               (IWSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSetOnKSSet < Element, ElementOps, Base > const&);

protected:

  static Boolean  isContained    (Element const&, void* env);
  static Boolean  isNotContained (Element const&, void* env);

};

#define IDefineGSetOnGKeySortedSet(GKSSet, GSet) \
template < class Element, class ElementOps > \
class GKSSet ## ForGSet : \
  public GKSSet < Element, Element, \
		  IOpsWithKey < Element, ElementOps > > { \
public: \
  GKSSet ## ForGSet (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    GKSSet < Element, Element, \
             IOpsWithKey < Element, ElementOps > > (n, b) {} \
}; \
template < class Element, class ElementOps > \
class GSet : \
  public IWSetOnKSSet < Element, ElementOps, \
		        GKSSet ## ForGSet < Element, ElementOps > > { \
public: \
  GSet (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    IWSetOnKSSet < Element, ElementOps, \
		   GKSSet ## ForGSet < Element, ElementOps > > (n, b) {} \
};

#include <isetkss.if>

#ifdef __IBMCPP__
#include <isetkss.c>
#endif

#endif
