/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _ISRKSB_H
#define _ISRKSB_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class Key, class ElementOps, class Base >
class IWSrtRelOnKSBag {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  public:
    Cursor (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWSrtRelOnKSBag             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IWSrtRelOnKSBag             (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   IWSrtRelOnKSBag < Element, Key, ElementOps, Base >&    operator =                  (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

                 ~IWSrtRelOnKSBag             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Boolean        contains                    (Element const&) const;

   Boolean        containsAllFrom             (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&) const;

   Boolean        locate                      (Element const&, ICursor&)
                                               const;

   Boolean        locateOrAdd                 (Element const&);

   Boolean        locateOrAdd                 (Element const&,
                                               ICursor&);

   Boolean        remove                      (Element const&);

   Boolean        operator ==                 (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&) const;

   Boolean        operator !=                 (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&) const;

   void           unionWith                   (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   void           intersectionWith            (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   void           differenceWith              (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   void           addUnion                    (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&,
                                               IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   void           addIntersection             (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&,
                                               IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   void           addDifference               (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&,
                                               IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&);

   Key const&     key                         (Element const&) const;

   Boolean        containsElementWithKey      (Key const&) const;

   Boolean        containsAllKeysFrom         (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&) const;

   Boolean        locateElementWithKey        (Key const&, ICursor&)
                                               const;

   Boolean        replaceElementWithKey       (Element const&);

   Boolean        replaceElementWithKey       (Element const&,
                                               ICursor&);

   Boolean        locateOrAddElementWithKey   (Element const&);

   Boolean        locateOrAddElementWithKey   (Element const&,
                                               ICursor&);

   Boolean        addOrReplaceElementWithKey  (Element const&);

   Boolean        addOrReplaceElementWithKey  (Element const&,
                                               ICursor&);

   Boolean        removeElementWithKey        (Key const&);

   Element const& elementWithKey              (Key const&) const;

   Element&       elementWithKey              (Key const&);

   INumber        numberOfElementsWithKey     (Key const&) const;

   Boolean        locateNextElementWithKey    (Key const&,
                                               ICursor&) const;

   INumber        removeAllElementsWithKey    (Key const&);

   INumber        numberOfDifferentKeys       () const;

   Boolean        setToNextWithDifferentKey   (ICursor&) const;

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   Boolean        setToLast                   (ICursor&) const;

   Boolean        setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   Boolean        isFirst                     (ICursor const&) const;

   Boolean        isLast                      (ICursor const&) const;

   long           compare                     (IWSrtRelOnKSBag < Element, Key, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

protected:

  static Boolean  isContained    (Element const&, void* env);
  static Boolean  isNotContained (Element const&, void* env);

  static ElementOps cvElementOps;

};

#define IDefineGSortedRelationOnGKeySortedBag(GKSBag, GSrtRel) \
template < class Element, class Key, class ElementOps > \
class GSrtRel : \
  public IWSrtRelOnKSBag \
	   < Element, Key, ElementOps, \
             GKSBag < Element, Key, ElementOps > > { \
public: \
  GSrtRel (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    IWSrtRelOnKSBag \
      < Element, Key, ElementOps, \
        GKSBag < Element, Key, ElementOps > > (n, b) {} \
};

#include <isrksb.if>

#ifdef __IBMCPP__
#include <isrksb.c>
#endif

#endif
