/* Copyright (c) IBM Corp. 1992 */
 
#ifndef _ISSKSS_H
#define _ISSKSS_H

#include <iglobals.h>
#include <icursor.h>

template < class Element, class ElementOps, class Base >
class IWSrtSetOnKSSet {
  Base ivBase;
  class Cursor;
  friend class Cursor;
public:

  class Cursor : public Base::Cursor {
  public:
    Cursor (IWSrtSetOnKSSet < Element, ElementOps, Base > const& c)
    : Base::Cursor (c.ivBase) {}
  };

                  IWSrtSetOnKSSet             (INumber
                                               numberOfElements = 100,
                                               IBoundIndicator =
                                               IUnbounded);

                  IWSrtSetOnKSSet             (IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   IWSrtSetOnKSSet < Element, ElementOps, Base >&    operator =                  (IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

                 ~IWSrtSetOnKSSet             ();

   Boolean        add                         (Element const&);

   Boolean        add                         (Element const&,
                                               ICursor&);

   void           addAllFrom                  (IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   Element const& elementAt                   (ICursor const&) const;

   Element&       elementAt                   (ICursor const&);

   Element const& anyElement                  () const;

   void           removeAt                    (ICursor const&);

   INumber        removeAll                   (Boolean (*property)
                                               (Element const&, void*),
                                               void* additionalArgument = 0);

   void           replaceAt                   (ICursor const&,
                                               Element const&);

   void           removeAll                   ();

   Boolean        isBounded                   () const;

   INumber        maxNumberOfElements         () const;

   INumber        numberOfElements            () const;

   Boolean        isEmpty                     () const;

   Boolean        isFull                      () const;

   ICursor*       newCursor                   () const;

   Boolean        setToFirst                  (ICursor&) const;

   Boolean        setToNext                   (ICursor&) const;

   Boolean        allElementsDo               (Boolean (*function)
                                                  (Element&, void*),
                                               void* additionalArgument = 0);

   Boolean        allElementsDo               (IIterator <Element>&);

   Boolean        allElementsDo               (Boolean (*function)
                                               (Element const&, void*),
                                               void* additionalArgument = 0)
                                               const;

   Boolean        allElementsDo               (IConstantIterator
                                                  <Element>&) const;

   Boolean        contains                    (Element const&) const;

   Boolean        containsAllFrom             (IWSrtSetOnKSSet < Element, ElementOps, Base > const&) const;

   Boolean        locate                      (Element const&, ICursor&)
                                               const;

   Boolean        locateOrAdd                 (Element const&);

   Boolean        locateOrAdd                 (Element const&,
                                               ICursor&);

   Boolean        remove                      (Element const&);

   Boolean        operator ==                 (IWSrtSetOnKSSet < Element, ElementOps, Base > const&) const;

   Boolean        operator !=                 (IWSrtSetOnKSSet < Element, ElementOps, Base > const&) const;

   void           unionWith                   (IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   void           intersectionWith            (IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   void           differenceWith              (IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   void           addUnion                    (IWSrtSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   void           addIntersection             (IWSrtSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   void           addDifference               (IWSrtSetOnKSSet < Element, ElementOps, Base > const&,
                                               IWSrtSetOnKSSet < Element, ElementOps, Base > const&);

   void           removeFirst                 ();

   void           removeLast                  ();

   void           removeAtPosition            (IPosition);

   Element const& firstElement                () const;

   Element const& lastElement                 () const;

   Element const& elementAtPosition           (IPosition) const;

   Boolean        setToLast                   (ICursor&) const;

   Boolean        setToPrevious               (ICursor&) const;

   void           setToPosition               (IPosition,
                                               ICursor&) const;

   Boolean        isFirst                     (ICursor const&) const;

   Boolean        isLast                      (ICursor const&) const;

   long           compare                     (IWSrtSetOnKSSet < Element, ElementOps, Base > const&,
                                               long (*comparisonFunction)
                                                  (Element const&,
                                                  Element const&)) const;

protected:

  static Boolean  isContained    (Element const&, void* env);
  static Boolean  isNotContained (Element const&, void* env);

};

#define IDefineGSortedSetOnGKeySortedSet(GKSSet, GSSet) \
template < class Element, class ElementOps > \
class GKSSet ## ForGSSet : \
  public GKSSet < Element, Element, \
		  IOpsWithKey < Element, ElementOps > > { \
public: \
  GKSSet ## ForGSSet (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    GKSSet < Element, Element, \
             IOpsWithKey < Element, ElementOps > > (n, b) {} \
}; \
template < class Element, class ElementOps > \
class GSSet : \
  public IWSrtSetOnKSSet < Element, ElementOps, \
		           GKSSet ## ForGSSet < Element, ElementOps > > { \
public: \
  GSSet (INumber n = 100, IBoundIndicator b = IUnbounded) : \
    IWSrtSetOnKSSet < Element, ElementOps, \
		      GKSSet ## ForGSSet < Element, ElementOps > > (n, b) {} \
};

#include <isskss.if>

#ifdef __IBMCPP__
#include <isskss.c>
#endif

#endif
