#ifndef _ITEXTCTL_
  #define _ITEXTCTL_
/*******************************************************************************
* FILE NAME: itextctl.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     Itextctl - This is an abstract base class for all text control window    *
*                classes.                                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   Licensed Materials - Property of IBM                                       *
*   (C) Copyright IBM Corporation 1992, 1993                                   *
*   All Rights Reserved                                                        *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
* $Log:   R:/IBMCLASS/IBASECTL/VCS/ITEXTCTL.HPV  $
//
//   Rev 1.10   25 Oct 1992 17:06:22   nunn
//changed library name to ICLUI
//
//   Rev 1.9   25 Oct 1992 10:58:14   boezeman
//Add documentation and converted file to skeleton.hpp format.
//
//   Rev 1.8   21 Oct 1992 14:32:22   greg
//Removed virtual inheritance

      Rev 1.7   13 Oct 1992 21:22:42   tsuji
   Correct comments around log directive.

      Rev 1.6   13 Oct 1992 20:58:04   tsuji
   Correct log directive, remove IBM Confidential classification.

      Rev 1.5   13 Oct 1992 19:03:50   tsuji
   Remove no-op virtual destructor, add Inherited typdef, change setTextColor
   back to virtual function.
*******************************************************************************/
#ifndef _ICONTROL_
  #include <icontrol.hpp>
#endif

// Forward declarations for other classes:
class ITextControl;  /* txtctl */
class IString;
class IResourceId;
#ifndef _IBITFLAG_
  #include <ibitflag.hpp>
#endif


//class ITextControl : public virtual IControl
class ITextControl : public IControl  {
/*******************************************************************************
* This is an abstract base class for all text control window classes.  It      *
* provides operations to manipulate the text string.  Controls which display   *
* a single text string are subclassed from this abstract base class.           *
*                                                                              *
* This class cannot be constructed directly.                                   *
*                                                                              *
* EXAMPLE:                                                                     *
*   <sample code>                                                              *
*******************************************************************************/
typedef IControl
  Inherited;
public:

/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There is 1 way to construct instances of this class:                         |
|   1. default                                                                 |
------------------------------------------------------------------------------*/
  ITextControl() {;}
                                  // need pure virtual function later

/*------------------------ QUERY/SET TEXT STRING -------------------------------
|  setText  -  Sets the control window text.                                   |
|  text     -  Returns the control window text.                                |
------------------------------------------------------------------------------*/
void
  setText(const char* text),
  setText(const IResourceId& text);
IString
  text() const;

/*------------------------ QUERY/SET TEXT STRING -------------------------------
|  setTextColor -  This operation changes the foreground color of the text.    |
------------------------------------------------------------------------------*/
virtual void
  setTextColor(const IColor& color);

private:
  ITextControl(const ITextControl&);
//    ITextControl& operator=(const ITextControl&);
};  // class ITextControl
#endif  // _ITEXTCTL_
