/* Copyright (c) IBM Corp. 1992 */
/*--------------------------------------------------------------------------*\
*                                                                            *
| Example of using the Building Block ISet.                                  |
*                                                                            *
\*--------------------------------------------------------------------------*/

#include <iostream.h>

/*---------------------- Building Block specification ----------------------*\
*                                                                            *
| A Set is specified.                                                        |
| It is based on a Sorted Mapping.                                           |
| The Sorted Mapping is based on a Linked Sequence.                          |
| The elements stored in the Set are integers.                               |
*                                                                            *
\*--------------------------------------------------------------------------*/

#include <iset.h>              //  ISetOnSortedMapping

typedef ISet <int> IntSet;

/*--------------------------------------------------------------------------*\
* Iterator                                                                   *
\*--------------------------------------------------------------------------*/
class PrintClass : public IIterator<int>
{
  public:
    virtual Boolean applyTo(int& i)
      { cout << " " << i << " "; return True;}
};


/*--------------------------------------------------------------------------*\
* Local prototype                                                            *
\*--------------------------------------------------------------------------*/
void    List(char *, IntSet &);

/*--------------------------------------------------------------------------*\
* Main program                                                               *
\*--------------------------------------------------------------------------*/
int main ()
{
   IntSet odd, prime;
   IntSet oddPrime, evenPrime;

   int One = 1, Two = 2, Three = 3, Five = 5, Seven = 7, Nine = 9;

   cout << "\n*** Example of ISet use ***\n";

// Fill odd set with odd integers < 10
   odd.add( One );
   odd.add( Three );
   odd.add( Five );
   odd.add( Seven );
   odd.add( Nine );
   List("Odds less than 10:", odd);

// Fill prime set with primes < 10
   prime.add( Two );
   prime.add( Three );
   prime.add( Five );
   prime.add( Seven );
   List("Primes less than 10:", prime);

// Intersect 'Odd' and 'Prime' to give 'OddPrime'
   oddPrime.addIntersection( odd, prime);
   List("Odd primes less than 10:", oddPrime);

// Subtract all 'Odd' from 'Prime' giving 'EvenPrime'
   evenPrime.addDifference( prime, oddPrime);
   List("Even primes less than 10:", evenPrime);

   cout << "\n*** End of example ***\n";

   return(0);
}

/*--------------------------------------------------------------------------*\
* Local function                                                             *
\*--------------------------------------------------------------------------*/

void List(char *Message, IntSet &anIntSet)
{
   PrintClass Print;

   cout << Message;
   anIntSet.allElementsDo(Print);
   cout << "\n";
}

