/*Ŀ*/
/*                                                                          */
/* PROGRAM NAME: PMSPY                                                      */
/* -------------                                                            */
/*  A PM program that is used to look at or 'spy' on the message queue of   */
/*  other PM applications windows.                                          */
/*                                                                          */
/* COPYRIGHT:                                                               */
/* ----------                                                               */
/*  Copyright (C) International Business Machines Corp., 1991,1992.         */
/*                                                                          */
/* DISCLAIMER OF WARRANTIES:                                                */
/* -------------------------                                                */
/*  The following [enclosed] code is sample code created by IBM Corporation.*/
/*  This sample code is not part of any standard IBM product and is provided*/
/*  to you solely for the purpose of assisting you in the development of    */
/*  your applications.  The code is provided "AS IS", without warranty of   */
/*  any kind.  IBM shall not be liable for any damages arising out of your  */
/*  use of the sample code, even if they have been advised of the           */
/*  possibility of such damages.                                            */
/*                                                                          */
/* For details on what this program does etc., please see the PMSPY.C file. */
/*                                                                          */
/**/

/*Ŀ*/
/* PMSPYFIL.C                                                               */
/*                                                                          */
/* Message Filter Dialog Procedure                                          */
/**/

#include "pmspy.h"

/*Ŀ*/
/* SetColors                                                                */
/*                                                                          */
/* - common subroutine to 'set up' the Color listbox in various dialogs     */
/**/

VOID SetColors( HWND       hwnd,
                PSPY_DATA  pSpyData )

{
 register USHORT i;

 /* Add all colors to the list in 'asis' order */

 for( /* Initialize */  i = 0;              /* start @ first */
      /* Terminate  */  i < Color_Total;    /* stop at End-Of-Table */
      /* Iterate    */  i++                 /* try the next */
    )
 {
   WinSendDlgItemMsg(hwnd,                           /* Window Handle  */
                     ID_LB_COLORS,                   /* Listbox ID     */
                     LM_INSERTITEM,                  /* Insert item    */
                     (MPARAM) LIT_END,               /* At the End     */
                     (MPARAM) ExternalColorTranslation[pSpyData->LogicalColorTranslation[i].iExtColor].pszClrName);
 }
}

/*Ŀ*/
/* GetColor                                                                 */
/*                                                                          */
/* - common subroutine to 'get the value' of the Color                      */
/*   selected by the user from the color LISTBOX                            */
/**/

MSG_COLOR GetColor( HWND       hwnd,
                    PSPY_DATA  pSpyData )

{
  register  SHORT     curItem;                     /* current list item */

  return( (curItem = (SHORT) WinSendDlgItemMsg(hwnd,               /* Dialog handle        */
                                               ID_LB_COLORS,       /* Listbox ID           */
                                               LM_QUERYSELECTION,  /* Query ID of selected */
                                               (MPARAM)LIT_FIRST,
                                               (MPARAM)NULL)) == LIT_NONE

          ? Color_Asis                     /* none selected, so leave ASIS */
          : pSpyData->LogicalColorTranslation[curItem].pbValue
        );
}


 MRESULT FilterDlgProc(          HWND    hwnd,
                        register ULONG   message,
                                 MPARAM  lParam1,
                                 MPARAM  lParam2 )

  {

     /*Ŀ*/
     /* Local Variables                                                     */
     /**/

     MRESULT            dpResult = NULL;
     register USHORT    pbSelected;
     CHAR               sFilterItem[100];

     static PSPY_DATA   pSpyData = NULL;

    /*Ŀ*/
    /* Process the message                                                  */
    /**/

    switch (message)                                    /* Switch off of the message ID */
      {

        case WM_COMMAND:

          switch( pbSelected = LOUSHORT( lParam1 ) )
          {

              /*Ŀ*/
              /* Process items marked for exclusion/inclusion/deletion      */
              /**/

              case ID_PB_F_DELETE:
              case ID_PB_F_EXCLUDE:
              case ID_PB_F_INCLUDE:

              {
                PMSG_ITEM     pMsg;                                   /* Points to element in chain   */

                USHORT        curItem,                                /* current list item */
                              nSelected;                              /* Number of items selected     */

                MSG_COLOR     color = GetColor( hwnd, pSpyData);

                for(/* Initialize */ nSelected = 0,                    /* Zero out vars                */
                                     curItem   = (USHORT)
                                                 WinSendDlgItemMsg(    /* Query 1st selection          */
                                                 hwnd,                 /*   Dialog handle              */
                                                 IDD_FILTER,           /*   Listbox ID                 */
                                                 LM_QUERYSELECTION,    /*   Query ID of selected       */
                                                 (MPARAM)LIT_FIRST,
                                                 (MPARAM)NULL);

                    /* Terminate  */ curItem != (USHORT)LIT_NONE;

                    /* Iterate    */ nSelected++,
                                     curItem   = (USHORT)
                                                 WinSendDlgItemMsg(    /* Query next selection         */
                                                 hwnd,                 /*   Dialog handle              */
                                                 IDD_FILTER,           /*   Listbox ID                 */
                                                 LM_QUERYSELECTION,    /*   Query ID of selected       */
                                                 (MPARAM)curItem,
                                                 (MPARAM)NULL)
                   )
                {

                  pMsg = (PMSG_ITEM)
                         WinSendDlgItemMsg( hwnd,
                                            IDD_FILTER,
                                            LM_QUERYITEMHANDLE, /*   Message                    */
                                            (MPARAM) curItem,   /*   Item to get handle of      */
                                            (MPARAM) NULL);     /*   Reserved                   */

                  switch( pbSelected )
                  {
                    case ID_PB_F_DELETE:
                         DeleteMsg(pSpyData, pMsg->Msg);
                         break;

                    case ID_PB_F_EXCLUDE:
                         IncludeMsg(pSpyData, pMsg->Msg, FALSE, color);
                         break;

                    case ID_PB_F_INCLUDE:
                         IncludeMsg(pSpyData, pMsg->Msg, TRUE, color);
                         break;
                  }
                }

                if (nSelected == 0)  /* Selections? */
                {
                  WinAlarm(HWND_DESKTOP, WA_ERROR);

                  break;                                   /* DO NOT end dialog */
                }
              }
              /*Ŀ*/
              /* *** FALL THROUGH TO END DIALOG ***                         */
              /**/

              /*Ŀ*/
              /* User cancelled exclusion/inclusion                         */
              /**/

              case DID_CANCEL:                          /* CANCEL was selected          */

                  WinDismissDlg( hwnd,                  /* Close dialog */
                                 0 );                   /* WinDlgBox return */

                break;                                  /* Break DID_CANCEL             */
          }

            break;                                      /* Break WM_COMMAND             */

        /*Ŀ*/
        /* WM_CLOSE:                                                        */
        /* - Call WinDismissDlg to close the dialog box                     */
        /**/

        case WM_CLOSE:                                  /* Close Dialog Box.    */

          WinDismissDlg( hwnd, 0);

          break;

        /*Ŀ*/
        /* WM_INITDLG:                                                      */
        /* - Init pointer to data pointed to by WinDlgBox                   */
        /* - Init global handle of this dlg                                 */
        /*                                                                  */
        /* - MP2 = PSPY_DATA                                                */
        /**/

        case WM_INITDLG:
             {
               register PMSG_ITEM  pMsg;
               register USHORT     list_index;

               /*Ŀ*/
               /* Locate "SPY" data to utilize                              */
               /**/

               pSpyData = (PSPY_DATA) lParam2;

               /*Ŀ*/
               /* Add all Messages to list box                              */
               /**/

               for( /* Initialize */  pMsg = ProcessFirstMsg(pSpyData); /* start @ first MSG */
                    /* Terminate  */  pMsg != NULL;                     /* stop at End-Of-Table */
                    /* Iterate    */  pMsg = ProcessNextMsg(pSpyData)   /* try the next MSG */
                  )
               {
                 sprintf(sFilterItem,
                         pMsg->Include ? Strings[IDS_FMT_MSG_INCLUDED]
                                       : Strings[IDS_FMT_MSG_EXCLUDED],
                         pMsg->pDesc,                     /* wording of Msg */
                         pMsg->Msg                        /* value   of Msg */
                        );

                 list_index = (USHORT) WinSendDlgItemMsg(      /* Send a message to the list   */
                               hwnd,                           /*   Window Handle              */
                               IDD_FILTER,
                               LM_INSERTITEM,                  /*   Insert item                */
                               (MPARAM) LIT_SORTASCENDING,     /*   Sort                       */
                               (MPARAM) sFilterItem);          /*   Text                       */

                 WinSendDlgItemMsg(                           /* Send a message to the list   */
                               hwnd,                          /*   Window Handle              */
                               IDD_FILTER,
                               LM_SETITEMHANDLE,              /*   Message                    */
                               (MPARAM)list_index,            /*   Index of list item         */
                               (MPARAM)pMsg);                 /*   Handle of list item        */

               /*Ŀ*/
               /* automatically "select" this message if being INCLUDED     */
               /*

               if ( pMsg->Include )
                 WinSendDlgItemMsg(hwnd,
                                   IDD_FILTER,
                                   LM_SELECTITEM,
                                   MPFROMSHORT(list_index),
                                   MPFROMSHORT(TRUE) );       */

               }

               SetColors( hwnd, pSpyData );

               CenterDialog(hwnd);               /* Center the Dialog */

             }

          break;

        /*Ŀ*/
        /* We don't need to handle any other messages...                    */
        /*                                                                  */
        /* If this isn't an IPF message, let PM do it's default "thing"     */
        /**/

        default:
             if ( !HandleIPF(hwnd, message, lParam1, lParam2, &dpResult) )
               dpResult = WinDefDlgProc(hwnd,       /* Dialog Handle                */
                                        message,    /* Message                      */
                                        lParam1,    /* First parameter for message. */
                                        lParam2);   /* Second parameter for message.*/

      } /* End MSG switch */

   /*Ŀ*/
   /* Exit                                                                  */
   /**/

   return( dpResult );
}
