/*Ŀ*/
/*                                                                          */
/* PROGRAM NAME: PMSPY                                                      */
/* -------------                                                            */
/*  A PM program that is used to look at or 'spy' on the message queue of   */
/*  other PM applications windows.                                          */
/*                                                                          */
/* COPYRIGHT:                                                               */
/* ----------                                                               */
/*  Copyright (C) International Business Machines Corp., 1991,1992.         */
/*                                                                          */
/* DISCLAIMER OF WARRANTIES:                                                */
/* -------------------------                                                */
/*  The following [enclosed] code is sample code created by IBM Corporation.*/
/*  This sample code is not part of any standard IBM product and is provided*/
/*  to you solely for the purpose of assisting you in the development of    */
/*  your applications.  The code is provided "AS IS", without warranty of   */
/*  any kind.  IBM shall not be liable for any damages arising out of your  */
/*  use of the sample code, even if they have been advised of the           */
/*  possibility of such damages.                                            */
/*                                                                          */
/* For details on what this program does etc., please see the PMSPY.C file. */
/*                                                                          */
/**/

/*Ŀ*/
/* PMSPYOW.C                                                                */
/*                                                                          */
/* Dialog procedure to display and select an OBJECT window for SPYing       */
/**/

#include "pmspy.h"                      /* Resource symbolic identifiers*/

#define  SYSTEM_OBJECT_WINDOW_ID '#'    /* first char of SYSTEM object class name */

typedef struct
{
  HWND       hwnd;
  char       szClass[40];
  PID        pidCreator;
  TID        tidCreator;
  CLASSINFO  classInfo;
  HMQ        hmq;

} OW_INFO, * POW_INFO;

/*Ŀ*/
/* Arg...these are global variables!                                        */
/**/

BOOL   defWantSYSTEM  = FALSE;

/*Ŀ*/
/* Routine to add one OBJECT window to the LISTBOX                          */
/**/
static void AddOneObjectToList( HWND   hwndList,      /* window of LISTBOX */
                                USHORT idList,        /* id     of LISTBOX */
                                HWND   hwndObject,    /* window of OBJECT */
                                USHORT depth,         /* "depth" of hierarchy */
                                BOOL   fWantSystem)   /* want SYSTEM object windows? */

{
  USHORT   list_index;

  OW_INFO  objData;

  POW_INFO pOW;

  char     szDepth[40],
           szData[128];

  PSZ      pszClass;

  memset(&objData, 0, sizeof(objData) );

  objData.hwnd = hwndObject;

  /* generate window depth guage */

  if (depth != 0)
    memset(szDepth, '*', depth);

  szDepth[depth] = 0;

  WinQueryClassName(hwndObject, sizeof(objData.szClass), objData.szClass);

  WinQueryClassInfo(WinQueryAnchorBlock(hwndList),
                    objData.szClass,
                    &objData.classInfo);

  WinQueryWindowProcess(hwndObject, &objData.pidCreator, &objData.tidCreator);

  objData.hmq = (HMQ)WinQueryWindowULong(hwndObject, QWL_HMQ);

  /* if this IS a system class and we don't want'em, ignore it! */

  if ( fWantSystem || (objData.szClass[0] != SYSTEM_OBJECT_WINDOW_ID) )
  {

    pszClass = TranslateClassName( objData.szClass );  /* translate SYSTEM */

    if ( pszClass != objData.szClass )  /* Class name translated? */
      sprintf( szData,                  /* yes... */
               "Class: %s (%s), Hwnd: 0x%lX, Hmq: 0x%lX, Parent: %u/%u",
               objData.szClass,
               pszClass,
               hwndObject,
               objData.hmq,
               objData.pidCreator,
               objData.tidCreator
             );
    else                                /* no... */
      sprintf( szData,
               "Class: %s, Hwnd: 0x%lX, Hmq: 0x%lX, Parent: %u/%u",
               objData.szClass,
               hwndObject,
               objData.hmq,
               objData.pidCreator,
               objData.tidCreator
             );

    pOW = (POW_INFO) AllocateMemory( sizeof( OW_INFO) );

    *pOW = objData;

    list_index = (USHORT) WinSendDlgItemMsg( /* Send a message to the list   */
                  hwndList,                  /*   Window Handle              */
                  idList,
                  LM_INSERTITEM,             /*   Insert item                */
                  (MPARAM)(fWantSystem
                           ? LIT_SORTDESCENDING /*Sort*/
                           : LIT_SORTASCENDING),
                  (MPARAM) szData);          /*   Text                       */

    WinSendDlgItemMsg(                       /* Send a message to the list   */
                  hwndList,                  /*   Window Handle              */
                  idList,
                  LM_SETITEMHANDLE,          /*   Message                    */
                  (MPARAM)list_index,        /*   Index of list item         */
                  (MPARAM)pOW);              /*   Handle of list item        */
  } /* endif */
}

/*Ŀ*/
/* Routine to recursively add all OBJECT windows to the LISTBOX             */
/**/
static void AddObjectChildrenToList(HWND   hwndList,      /* window of LISTBOX */
                                    USHORT idList,        /* id     of LISTBOX */
                                    HWND   hwndObject,    /* window of OBJECT */
                                    USHORT depth,         /* "depth" of hierarchy */
                                    BOOL   fWantSystem)   /* want SYSTEM object windows? */

{
  HWND     hwndChild;

  HENUM    hEnum = WinBeginEnumWindows( hwndObject );

  /* process all child windows of this window */

  for( /* Initialize */ hwndChild  = WinGetNextWindow(hEnum);
       /* While      */ hwndChild != NULLH;
       /* Iterate    */ hwndChild  = WinGetNextWindow(hEnum) )
  {
     AddOneObjectToList( hwndList,             /* window of LISTBOX */
                         idList,               /* id     of LISTBOX */
                         hwndChild,            /* window of OBJECT */
                         depth,                /* "depth" of hierarchy */
                         fWantSystem);         /* want SYSTEM object windows? */

     AddObjectChildrenToList( hwndList,        /* window of LISTBOX */
                              idList,          /* id     of LISTBOX */
                              hwndChild,       /* window of OBJECT */
                              depth + 1,       /* "depth" of hierarchy */
                              fWantSystem);    /* want SYSTEM object windows? */

     WinLockWindowUpdate(hwndChild, FALSE);    /* unLOCK the window */
  }

  WinEndEnumWindows( hEnum );
}

/*Ŀ*/
/* Routine to delete all existing OBJECT windows in the listbox             */
/*                                                                          */
/* - for performance and a visually 'nice' appear, we'll delete             */
/*   list items at once (after dealing with each's HANDLE data)             */
/**/
static VOID DeleteList(         HWND     hwndDlg,   /* handle of DIALOG */
                       register USHORT   idList)    /* ID of listbox in DIALOG */

{
           PSZ    pData;              /* data for current list item */

  register USHORT uItem;              /* deletion loop counter */

  register USHORT uTotalItems = (USHORT)
                                WinSendDlgItemMsg(hwndDlg,
                                                  idList,
                                                  LM_QUERYITEMCOUNT,
                                                  NULL,NULL);

  /*Ŀ*/
  /* Delete HANDLE data for every item                                      */
  /**/

  for( /* Initialization */  uItem = 0;                /* start @ first Item */
       /* While          */  uItem < uTotalItems;      /* while not all processed */
       /* Iterate        */  uItem++)                  /* to the next Item */
  {
    /*Ŀ*/
    /* Obtain item's "handlized" data and free allocated memory             */
    /**/

    pData = (PSZ) WinSendDlgItemMsg( hwndDlg,
                                     idList,
                                     LM_QUERYITEMHANDLE, /* give me HANDLE */
                                     (MPARAM)uItem,      /* current Item # */
                                     NULL);              /* Reserved */
    if (pData != NULL)
      FreeMemory( pData );
  }

  /*Ŀ*/
  /* Delete ALL item(s) in list at once (quick and more visually pleasing   */
  /* than do one at a time in the above loop)                               */
  /**/

  WinSendDlgItemMsg( hwndDlg,
                     idList,
                     LM_DELETEALL,     /* Delete ALL Item */
                     NULL,             /* Reserved */
                     NULL);            /* Reserved */
}

/*Ŀ*/
/* Routine to generate the list of OBJECT windows                           */
/**/
static USHORT GenerateList( HWND     hwndDlg,      /* handle of DIALOG */
                            USHORT   idList,       /* ID of listbox in DIALOG */
                            BOOL     WantSYSTEM)   /* want SYSTEM object windows? */

{
  register USHORT uNumberOfWindows;

  /*Ŀ*/
  /* First of all, "lock" the ListBox to minimize bad visual                */
  /* effect from the deletion and insertion of the VIEWs...                 */
  /**/

  WinLockWindowUpdate(HWND_DESKTOP,                      /* Desktop */
                      WinWindowFromID(hwndDlg,idList));  /* Lockee */

  /* start by erasing any existing OBJECT windows in the listbox */

  DeleteList( hwndDlg,       /* window of LISTBOX */
              idList);       /* id     of LISTBOX */

  /* start with HWND_OBJECT and add all OBJECT windows */

  AddOneObjectToList( hwndDlg,          /* window of LISTBOX */
                      idList,           /* id     of LISTBOX */
                      HWND_OBJECT,      /* window of OBJECT */
                      0,                /* "depth" of hierarchy */
                      WantSYSTEM);      /* determine if SYSTEM inc'ed */

  AddObjectChildrenToList( hwndDlg,     /* window of LISTBOX */
                           idList,      /* id     of LISTBOX */
                           HWND_OBJECT, /* parent window */
                           1,           /* "depth" of hierarchy */
                           WantSYSTEM); /* determine if SYSTEM inc'ed */

  /*Ŀ*/
  /* Finally, "unlock" the ListBox so it is only redrawn once               */
  /**/
  WinLockWindowUpdate(HWND_DESKTOP,  /* Desktop */
                      NULLH);         /* Re-enable ListBox */

  uNumberOfWindows = (USHORT)WinSendDlgItemMsg(hwndDlg,
                                               idList,
                                               LM_QUERYITEMCOUNT,
                                               NULL, NULL);

  if ( (!WantSYSTEM) && (uNumberOfWindows == 0) )
  {
    MsgDisplay( hwndDlg,
                swcntrl.szSwtitle,
                "No application OBJECT windows were found!",
                0,
                MB_CUACRITICAL | MB_CANCEL
              );
  }

  return( uNumberOfWindows );
}

/*Ŀ*/
/* Internal Function: DismissDialog                                         */
/*                                                                          */
/*                                                                          */
/* Function:                                                                */
/*                                                                          */
/*   This module: 1) sets the flag indicating if entered data should        */
/*                   be used by invoker of dialog                           */
/*                2) POSTs message to invoker of dialog                     */
/*                3) Delete all LISTBOX items                               */
/*                4) Dismisses the dialog                                   */
/*                                                                          */
/* Input:                                                                   */
/*       DlgHwnd.......dialog handle to dismiss                             */
/*       DlgData.......pointer to dialog's PNOTES_DATA structure            */
/*       DlgUse........indicator if entered NOTES data should be used       */
/*                                                                          */
/* Output:                                                                  */
/*                                                                          */
/*   - N O N E                                                              */
/**/

static void DismissDialog(HWND              hwndDlg,
                          POW_DATA          pDlgData,
                          register BOOL     fDlgUse)

{
  /*Ŀ*/
  /* Set the flag for caller's interogation                                 */
  /**/

  pDlgData->fUseData = fDlgUse;

  /*Ŀ*/
  /* Post REPLY message to caller                                           */
  /**/

  WinPostMsg(pDlgData->hwndNotify,
             pDlgData->uNotifyMsg,
             MPFROMP(pDlgData),
             MPFROMP(NULL) );

  /*Ŀ*/
  /* Delete all list items                                                  */
  /**/

  DeleteList(hwndDlg, ID_OW_LIST);

  /*Ŀ*/
  /* Dismiss the dialog                                                     */
  /**/

  WinDismissDlg(hwndDlg, fDlgUse);        /* Close the Dialog box         */
}

  /*Ŀ*/
  /* Dialog Procedure for displaying list of OBJECT windows                 */
  /**/

 MRESULT ObjectDlgProc(          HWND    hwnd,
                        register ULONG   message,
                                 MPARAM  lParam1,
                                 MPARAM  lParam2 )

{

  /*Ŀ*/
  /* Local Variables                                                        */
  /**/

  MRESULT   dpResult  = NULL;
  POW_INFO  pOW;

  /*Ŀ*/
  /* Obtain pointer to this dialog's instance data                          */
  /**/

  POW_DATA pData = (POW_DATA) WinQueryWindowPtr(hwnd, QWL_USER);

    /*Ŀ*/
    /* Process the message                                                  */
    /**/

    switch (message)                       /* Switch off of the message ID */
      {
        /*Ŀ*/
        /* One of the PUSHBUTTON has been used...                           */
        /**/

        case WM_COMMAND:

          switch( LOUSHORT( lParam1 ) )
          {
              /*Ŀ*/
              /* process the selected OBJECT window                         */
              /**/

              case ID_OW_PB_SPYON:

              {
                USHORT        curItem;                 /* current list item */

                if ( (curItem = (USHORT)
                                WinSendDlgItemMsg(      /* Query 1st selection          */
                                  hwnd,                 /*   Dialog handle              */
                                  ID_OW_LIST,           /*   Listbox ID                 */
                                  LM_QUERYSELECTION,    /*   Query ID of selected       */
                                  (MPARAM)LIT_FIRST,
                                  (MPARAM)NULL)) != LIT_NONE )
                {
                  pOW = (POW_INFO)
                         WinSendDlgItemMsg( hwnd,
                                            ID_OW_LIST,
                                            LM_QUERYITEMHANDLE, /*   Message                    */
                                            (MPARAM) curItem,   /*   Item to get handle of      */
                                            (MPARAM) NULL);     /*   Reserved                   */

                  /*Ŀ*/
                  /* Switch to SPYing this OBJECT window...                 */
                  /**/

                  pData->hwndOW = pOW->hwnd;
                  pData->hmqOW  = pOW->hmq;

                  DismissDialog(hwnd, pData, TRUE);  /* end dialog (do USE data) */
                }
                else
                  WinAlarm(HWND_DESKTOP, WA_ERROR);
              }
              break;

              /*Ŀ*/
              /* refresh the list with all OBJECT windows                   */
              /**/

              case ID_OW_PB_REFRESH_ALL:

                   GenerateList(hwnd, ID_OW_LIST, TRUE);

              break;

              /*Ŀ*/
              /* refresh the list with only non-system OBJECT windows       */
              /**/

              case ID_OW_PB_REFRESH_PGMR:

                   GenerateList(hwnd, ID_OW_LIST, FALSE);
              break;

              /*Ŀ*/
              /* User cancelled exclusion/inclusion                         */
              /**/

              case DID_CANCEL:                          /* CANCEL was selected          */

                   DismissDialog(hwnd, pData, FALSE);   /* DO NOT use USE data...       */

              break;                                    /* Break DID_CANCEL             */
            }

          break;                                        /* Break WM_COMMAND             */

        /*Ŀ*/
        /* WM_CLOSE:                                                        */
        /* - Call WinDismissDlg to close the dialog box                     */
        /**/

        case WM_CLOSE:

             DismissDialog(hwnd, pData, FALSE); /* DO NOT use USE data... */

             dpResult = MRFROMSHORT( TRUE );

        break;

         /*Ŀ*/
         /* WM_INITDLG:                                                     */
         /*                                                                 */
         /* - Load list with current set of OBJECT windows                  */
         /*                                                                 */
         /* - MP2 is dialog instance's data                                 */
         /**/

         case WM_INITDLG:
              /*Ŀ*/
              /* Save Dialog's instance data pointer (passed in MP2)        */
              /**/

              WinSetWindowPtr(hwnd, QWL_USER, pData = (POW_DATA)lParam2);

              /*Ŀ*/
              /* Setup the Dialog for processing...                         */
              /**/

              CenterDialog(hwnd);          /* Center the Dialog */

              GenerateList(hwnd, ID_OW_LIST, defWantSYSTEM);

          break;

         /*Ŀ*/
         /* We don't need to handle any other messages...                   */
         /*                                                                 */
         /* If this isn't an IPF message, let PM do it's default "thing"    */
         /**/

         default:
              if ( !HandleIPF(hwnd, message, lParam1, lParam2, &dpResult) )
                dpResult = WinDefDlgProc(hwnd, message, lParam1, lParam2 );

      }                                                 /* End switch                   */

   /*Ŀ*/
   /* Exit                                                                  */
   /**/

   return( dpResult );
}
