/*Ŀ*/
/*                                                                          */
/* PROGRAM NAME: PMSPY                                                      */
/* -------------                                                            */
/*  A PM program that is used to look at or 'spy' on the message queue of   */
/*  other PM applications windows.                                          */
/*                                                                          */
/* COPYRIGHT:                                                               */
/* ----------                                                               */
/*  Copyright (C) International Business Machines Corp., 1991,1992.         */
/*                                                                          */
/* DISCLAIMER OF WARRANTIES:                                                */
/* -------------------------                                                */
/*  The following [enclosed] code is sample code created by IBM Corporation.*/
/*  This sample code is not part of any standard IBM product and is provided*/
/*  to you solely for the purpose of assisting you in the development of    */
/*  your applications.  The code is provided "AS IS", without warranty of   */
/*  any kind.  IBM shall not be liable for any damages arising out of your  */
/*  use of the sample code, even if they have been advised of the           */
/*  possibility of such damages.                                            */
/*                                                                          */
/* For details on what this program does etc., please see the PMSPY.C file. */
/*                                                                          */
/**/

/*Ŀ*/
/* PMSPYUTL.C                                                               */
/*                                                                          */
/* Utility routines                                                         */
/**/

/*Ŀ*/
/* Includes                                                                 */
/**/

#include "pmspy.h"                      /* Resource symbolic identifiers*/

#include <ctype.h>

typedef struct {                        /* Memory Allocation control structure */

                 ULONG  nAllocs,
                        sAllocs,
                        nFrees;
               } MEM_DATA, *PMEM_DATA;

static MEM_DATA memStats = { 0, 0, 0};

/*Ŀ*/
/* Routine:    AllocateMemory                                               */
/*                                                                          */
/* Function:   Allocate dynamic memory (also, sets to all NULLs)            */
/**/

PVOID AllocateMemory(register USHORT uSizeWanted )

{
  PVOID p;

  /*** Attempt to allocate the requested memory ***/

  if ( (p = malloc( uSizeWanted )) == NULL)
  {
    MsgDisplay(HWND_DESKTOP,
               Strings[IDS_TITLE],
               Strings[IDS_MSG_ALLOCATE],
               0,
               MB_CUAWARNING | MB_CANCEL,
               uSizeWanted,
               memStats.nAllocs,
               memStats.sAllocs,
               memStats.nFrees);
  }
  else
  {
    /*** Update statistics ***/
    memStats.nAllocs++;
    memStats.sAllocs += uSizeWanted;

    /* zero out allocated area */
    memset(p, 0, uSizeWanted);
  }

  return( p );

}

/*Ŀ*/
/* Routine:    FreeMemory                                                   */
/*                                                                          */
/* Function:   Free memory obtained from AllocateMemory                     */
/**/

VOID FreeMemory(register PVOID p )

{
   if (p == NULL)
   {
    MsgDisplay(HWND_DESKTOP,
               Strings[IDS_TITLE],
               Strings[IDS_MSG_FREE],
               0,
               MB_CUAWARNING | MB_CANCEL,
               p,
               memStats.nAllocs,
               memStats.sAllocs,
               memStats.nFrees);
   }
   else
   {
     memStats.nFrees++;
     free( p );
   } /* endif */
}

/*Ŀ*/
/* Routine:    QueryMemory                                                  */
/*                                                                          */
/* Function:   Query memory statistics                                      */
/**/

VOID QueryMemory(register PMEM_DATA p )

{
  *p = memStats;                /* return COPY to caller */
}

/*Ŀ*/
/* Routine:    MsgDisplay                                                   */
/*                                                                          */
/* Function:   Display MessageBox with variable substitution                */
/**/

static BOOL bEnableHELP = FALSE;

USHORT MsgDisplay( HWND   hWnd,
                   PSZ    strCap,
                   PSZ    strFormat,
                   USHORT mb_id,
                   USHORT style,
                   ...)

{
  CHAR          MessageText[512];                       /* Message box output           */

  va_list       arg_ptr;                                /* Variable argument list ptr   */

  /*Ŀ*/
  /*                            MAINLINE                                    */
  /**/

  if ( bEnableHELP )                  /* add HELP button is necessary */
    style |= MB_HELP;

  va_start(arg_ptr, style);                     /* Start variable list access   */

  return( vsprintf(                             /* Format the message box output*/
                   MessageText,                 /*  Buffer to put it in         */
                   strFormat,                   /*  Format string               */
                   arg_ptr) == -1               /*  Parameters                  */

          ? MID_ERROR

          : WinMessageBox(                      /* Display message box          */
                           HWND_DESKTOP,        /*   Desktop handle             */
                           hWnd,                /*   Client window handle       */
                           MessageText,         /*   Main body of message text  */
                           strCap,              /*   Caption of Box             */
                           mb_id,               /*   ID of message box (Help)   */
                           style       |
                           MB_MOVEABLE |
                           MB_SYSTEMMODAL)      /*   Style of message box       */
        );
}

/*Ŀ*/
/* Routine:    MsgDisplay                                                   */
/*                                                                          */
/* Function:   Display MessageBox with variable substitution                */
/**/

VOID MsgDisplayShouldEnableHELP( VOID )

{
  bEnableHELP = TRUE;

}

/*Ŀ*/
/* Routine: ConvertToNumber                                                 */
/* ------------------------------------------------------------------------ */
/*                                                                          */
/* Purpose:   Convert an Ascii text string to its binary value using        */
/*            C language format conventions                                 */
/*                                                                          */
/*            a) decimal          dddd                                      */
/*            b) octal           0dddd                                      */
/*            c) hexidecimal    0xdddd                                      */
/*                                                                          */
/*            Complete Format: [<white space>]<number>                      */
/*                                                                          */
/*                                                                          */
/* Prototype: BOOL ConvertToNumber(PSZ   pAscii,                            */
/*                                 CTN  *pNumber,                           */
/*                                 CTN   lMin,                              */
/*                                 CTN   lMax);                             */
/*                                                                          */
/* Entry:     pAscii.....pointer to NULL terminated string to attempt to    */
/*                       convert to binary                                  */
/*            pNumeric...pointer to variable to receive converted value     */
/*            lMim.......minimum acceptable range value (inclusive)         */
/*            lMax.......maximum acceptable range value (inclusive)         */
/*                                                                          */
/* Exit:      TRUE.......conversion was successful                          */
/*                       ==> *pNumber contains converted value              */
/*                                                                          */
/*            FALSE......conversion was not successful                      */
/*                       ==> *pNumber is indeterminate                      */
/**/

BOOL ConvertToNumber(PSZ     pAscii,
                     PUSHORT pNumber,
                     USHORT  lMin,
                     USHORT  lMax)

{
  register BOOL  fOK = FALSE;                 /* start as a pessimist! */
           PSZ   pEnd;                        /* locates last char converted */

           ULONG ulConversion = strtoul(pAscii,
                                        &pEnd,
                                        0);     /* figure base out */

  /*Ŀ*/
  /* Determine if conversion was successful                                 */
  /**/

  if (*pEnd == 0)
  {
    /*Ŀ*/
    /* Conversion was successful, is number within allowed range?           */
    /**/

    *pNumber = LOUSHORT(ulConversion); /* convert from 32 to 16 bits */

    if ( (*pNumber >= lMin) && (*pNumber <= lMax) )
      fOK = TRUE;
  }

  return( fOK );
}

/*Ŀ*/
/* Routine: OpenPDTR                                                        */
/* -------------------------------------------------------------------------*/
/*                                                                          */
/* Purpose:   Open a Programmer Defined Text Resource (PDTR) for later      */
/*                                                                  READing */
/* Prototype: PPDTR_DATA openPDTR(PPDTR_DATA pData)                         */
/*                                                                          */
/* Entry:     pData......pointer to PDTR control structure                  */
/*                                                                          */
/* Exit:      !NULL......Open was successful                                */
/*                       ==> ready to Read/Eof/Close                        */
/*                                                                          */
/*             NULL......Open failed                                        */
/*                       ==> probable causes: - invalid hModule, typePDTR,  */
/*                                              or idPDTR value specified   */
/**/

PPDTR_DATA OpenPDTR(register PPDTR_DATA pData)

{
  register BOOL bOK = FALSE;

  if (DosGetResource(  pData->hModule,         /* Who to get from */
                       pData->typePDTR,        /* What Resource Type */
                       pData->idPDTR,          /* What Resource ID */
            (PPVOID) &(pData->offPDTR)) == 0)  /* Where placed */
  {
    pData->pTOF = pData->pRead = pData->offPDTR;

    bOK = TRUE;
  }

  return(bOK ? pData : NULL);
}

#define PDTR_EOF 0x1A                  /* End-Of-File byte */
#define PDTR_EOL 0xD                   /* End-Of-Line byte */

/*Ŀ*/
/* Routine: EofPDTR                                                         */
/* --------------------------------------------------------------------     */
/*                                                                          */
/* Purpose:   Determine if Open Programmer Defined Text Resource (PDTR)     */
/*            is at End-Of-File                                             */
/*                                                                          */
/* Prototype: BOOL EofPDTR(PPDTR_DATA pData)                                */
/*                                                                          */
/* Entry:     pData......pointer to PDTR control structure                  */
/*                                                                          */
/* Exit:      TRUE.......File is at EOF                                     */
/*                                                                          */
/*            FALSE......File is not at EOF                                 */
/**/

BOOL EofPDTR(register PPDTR_DATA pData)

{
  return( *pData->pRead == PDTR_EOF );
}

/*Ŀ*/
/* Routine: GetsPDTR                                                        */
/* -------------------------------------------------------------------------*/
/*                                                                          */
/* Purpose:   Get String from an Open Programmer Defined Text Resource(PDTR)*/
/*                                                                          */
/* Prototype: PSZ GetsPDTR(PPDTR_DATA pData, PSZ pString, USHORT lString)   */
/*                                                                          */
/* Entry:     pData......pointer to PDTR control structure                  */
/*            pString....pointer to area to put string                      */
/*            lString....length of pString area (must be ONE or more!)      */
/*                                                                          */
/* Exit:      NULL.......File is at EOF                                     */
/*           !NULL.......entry pString value                                */
/**/

/*Ŀ*/
/* PDTR format: <text....><CR><LF>                                          */
/*              <text....><CR><LF>                                          */
/*              <text....><CR><LF>                                          */
/*              <EOF>                                                       */
/**/

PSZ GetsPDTR(         PPDTR_DATA pData,
                      PSZ        pString,
             register USHORT     lString)

{
  register PSZ pWork;                           /* work in pString area */

  /* if at EOF, quit right away */

  if ( *pData->pRead == PDTR_EOF )
    return( NULL );

  /*Ŀ*/
  /* Copy line from SEGMENT until: 1) EOL reached                           */
  /*                               2) pString area filled                   */
  /**/
  for(/* Init */ pWork = pString;               /* where to put string */

      /* Term */ (*pData->pRead != PDTR_EOL) && /* quit @ EOL */
                 (lString       != 1);          /* or no room left */

      /* Iter */ pWork++,                       /* move to next string place */
                 pData->pRead++,                /* move READ pointer */
                 lString--                      /* adjust remaining size */
     )
    *pWork = *pData->pRead;

  pData->pRead += 2;   /* skip CR/LF */

  *pWork = 0;       /* make NULL terminated.... */

  return( pString );
}

/*Ŀ*/
/* Routine: ClosePDTR                                                       */
/* ----------------------------------------------------------------         */
/*                                                                          */
/* Purpose:   Close an Open Programmer Defined Text Resource (PDTR)         */
/*                                                                          */
/* Prototype: BOOL ClosePDTR(PPDTR_DATA pData)                              */
/*                                                                          */
/* Entry:     pData......pointer to PDTR control structure                  */
/*                                                                          */
/* Exit:      0..........PDTR close successful                              */
/*                                                                          */
/*            EOF........PDTR close unsuccessful                            */
/**/

int ClosePDTR(register PPDTR_DATA pData)

{
  return (DosFreeResource (pData->offPDTR) == 0 ? 0 : EOF);
}

/*Ŀ*/
/* Routine: FindWordInList                                                  */
/* ----------------------------------------------------------------------   */
/*                                                                          */
/* Purpose:   Find a NULL terminated "word" in a "word list"                */
/*                                                                          */
/* Prototype: USHORT FindWordInList(PSZ pWord, PWORD_LIST pWordList);       */
/*                                                                          */
/* Entry:     pWord......pointer to WORD to look for in pWordList           */
/*            pWordList..pointer to WORD list                               */
/*                                                                          */
/* Exit:      'value' for "word" (or value from End-Of-List marker saying   */
/*            that the "word" was not found                                 */
/**/

USHORT FindWordInList(PSZ pWord, PWORD_LIST pWordList)

{
  for( /* Init  */   /**** N O N E ****/;

       /* While */   (pWordList->ppWord != NULL) &&
                     (stricmp(pWord, *pWordList->ppWord) != 0);

       /* Iter  */   pWordList++ );

  return(pWordList->vWord );

}


/*Ŀ*/
/* Routine: UpdateSpyTitle                                                  */
/* --------------------------------------------------------                 */
/*                                                                          */
/* Purpose:   Update the title of the PMSPY frame                           */
/*                                                                          */
/* Prototype: VOID UpdateSpyTitle( HWND         hwndFrame,                  */
/*                                 SPY_TITLES   whichTitle,                 */
/*                                 ...);                                    */
/*                                                                          */
/* Entry:     hwndFrame....frame window                                     */
/*            whichTitle...desired title mode                               */
/*                                                                          */
/* Exit:      NONE, except title in frame updated                           */
/**/

VOID UpdateSpyTitle(HWND         hwndFrame,
                    SPY_TITLES   whichTitle,
                    ...)
{
  CHAR          szText[256];

  va_list       arg_ptr;                /* Variable argument list ptr */

  /*Ŀ*/
  /* Locate start of variable parm(s) passed to us                          */
  /**/

  va_start(arg_ptr, whichTitle);        /* Start variable list access */

  /*Ŀ*/
  /* Generate the new PMSPY title                                           */
  /**/
  switch( whichTitle )
  {
    /*Ŀ*/
    /* Display information about the "window" the POINTER is currently over */
    /**/
    case tSpyWho:
         vsprintf(szText, Strings[IDS_FMT_PTR_QUEUE], arg_ptr);
    break;

    /*Ŀ*/
    /* Change title to display "window" being SPYed                         */
    /**/
    case tWindow:
         vsprintf(szText, Strings[IDS_FMT_SPY_WINDOW], arg_ptr);
    break;

    /*Ŀ*/
    /* Change title to display "object" being SPYed                         */
    /**/
    case tObject:
         vsprintf(szText, Strings[IDS_FMT_SPY_OBJECT], arg_ptr);
    break;

    /*Ŀ*/
    /* Change title to display "queue" being SPYed                          */
    /**/
    case tQueue:
         vsprintf(szText, Strings[IDS_FMT_SPY_QUEUE], arg_ptr);
    break;

    /*Ŀ*/
    /* Change title back to PMSPY standard                                  */
    /**/

    default:
         strcpy(szText, swcntrl.szSwtitle);
  }

  /*Ŀ*/
  /* Update the PMSPY title                                                 */
  /**/

  WinSetWindowText(hwndFrame, szText);

  /*Ŀ*/
  /* LOG the new PMSPY title, if we need to                                 */
  /**/

  if ( bCurrentlyLogging )
    fprintf(pLogFile, "%s\n", szText);
}

/*Ŀ*/
/* Routine: TranslateClassName                                              */
/* ------------------------------------------------------------------------ */
/*                                                                          */
/* Purpose:  Translate WinQueryClassName to usable WC_* string              */
/*                                                                          */
/* Prototype: PSZ TranslateClassName(PSZ pszClassName)                      */
/*                                                                          */
/* Entry:     pszClassName....name from WinQueryClassName                   */
/*                                                                          */
/* Returns:   if a standard WC_* class, then pointer to usable string       */
/*                                      else pointer to original class name */
/**/

PSZ TranslateClassName(PSZ pszClassName)

{

           PSZ    pszTranslate;        /* final, translated class name */
  register USHORT i;

/*Ŀ*/
/* Translate table of WinClassName value to WC_* strings                    */
/**/

static struct
{ PSZ pszClassName,
      pszWC;   /*  WinQueryClassName    Human-useable value */
} Class2WC[] = { {"#1",                 "WC_FRAME"              },
                 {"#2",                 "WC_COMBOBOX"           },
                 {"#3",                 "WC_BUTTON"             },
                 {"#4",                 "WC_MENU"               },
                 {"#5",                 "WC_STATIC"             },
                 {"#6",                 "WC_ENTRYFIELD"         },
                 {"#7",                 "WC_LISTBOX"            },
                 {"#8",                 "WC_SCROLLBAR"          },
                 {"#9",                 "WC_TITLEBAR"           },
                 {"#A",                 "WC_MLE"                },

                 {"#10",                "WC_APPSTAT"            },
                 {"#11",                "WC_KBDSTAT"            },
                 {"#12",                "WC_PECIC"              },
                 {"#13",                "WC_DBE_KKPOPUP"        },

                 {"#20",                "WC_SPINBUTTON"         },
                 {"#25",                "WC_CONTAINER"          },
                 {"#26",                "WC_SLIDER"             },
                 {"#27",                "WC_VALUESET"           },
                 {"#28",                "WC_NOTEBOOK"           },

                 {"#32766",             "Desk Top"              },

                 {NULL,                 NULL                    }
               };

  /*Ŀ*/
  /* Translate all WC_* class names to something human understandable       */
  /**/

  for(/* Initialize */ pszTranslate =  NULL,
                       i = 0;
      /* While      */ pszTranslate == NULL;
      /* Iterate    */ i++)
  {
     /*Ŀ*/
     /* At end of translate table?  if so, use class name ASIS              */
     /**/
     if (Class2WC[i].pszClassName == NULL)
       pszTranslate = pszClassName;
     /*Ŀ*/
     /* Is this a WC_* class?  if so, use translation                       */
     /**/
     else if (strcmp(pszClassName, Class2WC[i].pszClassName) == 0)
       pszTranslate = Class2WC[i].pszWC;
  }

  /*Ŀ*/
  /* Return the translation result                                          */
  /**/

  return( pszTranslate );
}
/*Ŀ*/
/* Module Level Design                                                      */
/*                                                                          */
/*                                                                          */
/* Function:                                                                */
/*     To center a dialog in the middle of the DESKTOP area                 */
/*                                                                          */
/* Input:                                                                   */
/*     - Dialog's Window Handle                                             */
/*                                                                          */
/*                                                                          */
/* Output:                                                                  */
/*     - Returns:                                                           */
/*       a) TRUE........ no error detected (dialog nicely centered!)        */
/*       b) FALSE....... error detected (logged to RAS error log)           */
/*                                                                          */
/* Notes:                                                                   */
/*     -                                                                    */
/*                                                                          */
/* Logic:                                                                   */
/*     - get size of DESKTOP area (results are 'window' coordinates)        */
/*     - convert DESKTOP area to 'screen' coordinates                       */
/*     - get size of Dialog (results are 'window' coordinates)              */
/*     - convert Dialog area to 'screen' coordinates                        */
/*     - use WinSetPos to position the Dialog centered on the DESKTOP       */
/**/

BOOL CenterDialog(HWND hwndDialog)     /* Dialog window to center */

{
  /*Ŀ*/
  /* Local Variables                                                        */
  /**/

  BOOL     fOK      = FALSE;         /* start as a pessimist! */

  ERRORID  rcPM     = 0;             /* Return code */

  RECTL    rectlDesktop,             /* Rectl of DeskTop */
           rectlDialog;              /* Rectl of dialog */

  HAB      habDialog = WinQueryAnchorBlock(hwndDialog); /* HAB for dialog */

  /*Ŀ*/
  /* Note: using DO_ONCE_BLOCK for easy error recovery                      */
  /**/

  BEGIN_DO_ONCE_BLOCK

    /*Ŀ*/
    /* Gather information on the DESKTOP                                    */
    /*                                                                      */
    /* - get DESKTOP rectangle (in window coordinates)                      */
    /* - convert to screen coordinates                                      */
    /**/

    if ( !WinQueryWindowRect(HWND_DESKTOP, &rectlDesktop) ||

         !WinMapWindowPoints(hwndDialog,              /* From: Window coordinates */
                             HWND_DESKTOP,            /* To:   Desktop coordinates */
                             (PPOINTL)&rectlDesktop,  /* address(points) */
                             2)  ||                   /* number of points */

    /*Ŀ*/
    /* Gather information on the dialog                                     */
    /*                                                                      */
    /* - get DESKTOP rectangle                                              */
    /* - convert to window coordinates                                      */
    /**/

         !WinQueryWindowRect(hwndDialog, &rectlDialog) ||

         !WinMapWindowPoints(hwndDialog,              /* From: Window coordinates */
                             HWND_DESKTOP,            /* To:   Desktop coordinates */
                             (PPOINTL)&rectlDialog,   /* address(points) */
                             2) )
    {
      EXIT_DO_ONCE_BLOCK;
    }

    /*Ŀ*/
    /* Position Dialog in center of Desk Top                                */
    /*                                                                      */
    /* Centered X position = (Width (DeskTop) - Width (Dialog) ) / 2        */
    /* Centered Y position = (Height(DeskTop) - Height(Dialog) ) / 2        */
    /**/

    if ( WinSetWindowPos(hwndDialog,
                         HWND_TOP,                       /* Z_ORDER */
                         (SHORT)(                        /* X position */
                                 ((rectlDesktop.xRight - /*  Desktop height */
                                   rectlDesktop.xLeft) -
                                  (rectlDialog .xRight - /*  Dialog  height */
                                   rectlDialog .xLeft))/2
                                ),

                         (SHORT)(                        /* Y position */
                                 ((rectlDesktop.yTop   - /* Desktop width */
                                   rectlDesktop.yBottom) -
                                  (rectlDialog .yTop   - /* Dialog  width */
                                   rectlDialog .yBottom))/2
                                ),

                         (SHORT)NULL,
                         (SHORT)NULL,
                         SWP_MOVE | SWP_ZORDER) )        /* Flags */

       fOK = TRUE;         /* we're a success! */
    else
    {
      EXIT_DO_ONCE_BLOCK;
    }

  END_DO_ONCE_BLOCK;

 /*Ŀ*/
 /* Exit                                                                    */
 /**/

 return( fOK );
}
