/*Ŀ*/
/*                                                                          */
/* PROGRAM NAME: PMSPY                                                      */
/* -------------                                                            */
/*  A PM program that is used to look at or 'spy' on the message queue of   */
/*  other PM applications windows.                                          */
/*                                                                          */
/* COPYRIGHT:                                                               */
/* ----------                                                               */
/*  Copyright (C) International Business Machines Corp., 1991,1992.         */
/*                                                                          */
/* DISCLAIMER OF WARRANTIES:                                                */
/* -------------------------                                                */
/*  The following [enclosed] code is sample code created by IBM Corporation.*/
/*  This sample code is not part of any standard IBM product and is provided*/
/*  to you solely for the purpose of assisting you in the development of    */
/*  your applications.  The code is provided "AS IS", without warranty of   */
/*  any kind.  IBM shall not be liable for any damages arising out of your  */
/*  use of the sample code, even if they have been advised of the           */
/*  possibility of such damages.                                            */
/*                                                                          */
/* For details on what this program does etc., please see the PMSPY.C file. */
/*                                                                          */
/**/

/*Ŀ*/
/* PMSPYWIN.C                                                               */
/*                                                                          */
/* "Client Area" window Procedure for the PMSPY Main Window                 */
/**/

/*Ŀ*/
/* Includes                                                                 */
/**/
#include "pmspy.h"                      /* Resource symbolic identifiers*/
#include "pmspyipf.h"                   /* IPF panel identifiers*/

/*Ŀ*/
/* Localized items                                                          */
/**/

typedef struct                            /* template for owner draw item */
{
  CHAR        logCLR,                     /* logical color index */
              useHalfTone,                /* draw using DT_HALFTONE? ('1'=yes) */
              szMsg[L_NOTES_TEXT];        /* MSG to draw */

} HOOK_DATA, *PHOOKDATA;

/*Ŀ*/
/* Spy Window Procedure Data                                                */
/**/
static CHAR          szFileName[CCHMAXPATH],
                     szHelpFile[] = "PMSPY.HLP";

static SPY_DATA  spyData;
static PSPY_DATA pSpyData      =  &spyData;

static USHORT    NbrOfMessages = 0;

static HWND      hwndMsgList   = NULLH, /* Msg list scroll list window handle */
                 hwndFrame     = NULLH, /* Frame (Parent of this CLIENT window) */
                 hwndMenu      = NULLH, /* Frame's menu                       */
                 hwndIPF       = NULLH; /* IPF handle                         */

static LONG   lItemHeight;  /* height of an item returned by WM_MEASUREITEM   */
static SHORT  sNbrOfItems;  /* nbr of items in the list neede to fill client window */

/*Ŀ*/
/* PMSPY INI file data items                                                */
/**/

static CHAR szINI[]            = "PMSPY.INI",
            szAPPL[]           = "PMSPY",

            szDefFont[]        = "10.System Monospaced",

            szKeyFont[]        = "FONT",
            szKeyPosX[]        = "POS_X",
            szKeyPosY[]        = "POS_Y",
            szKeySizeCX[]      = "SIZE_CX",
            szKeySizeCY[]      = "SIZE_CY";

INI_DATA iniAgent = {
                      "10.System Monospaced",
                      { 0, 0, 0, 0}
                    };

/*Ŀ*/
/* Write Agent's PMSPY data to INI file                                     */
/**/

static BOOL WriteSpyINI(HWND      hFrame,   /* HWND(Agent's frame) */
                        SPYEE     Agent,    /* one-origin "agent" */
                        PINI_DATA pIni)     /* INI data to write */
{
  CHAR szAgent[32],
       szData[128];

  HINI hINI          = NULLH;
  BOOL bOK           = FALSE;

  /*Ŀ*/
  /* Open the PMSPY "profile"                                               */
  /**/

  if ( (hINI = PrfOpenProfile( WinQueryAnchorBlock(hFrame),
                               szINI)) != NULLH )
  {
    /*Ŀ*/
    /* Determine which "agent" this data is for                             */
    /**/

    sprintf(szAgent, Strings[IDS_FMT_AGENT], Agent);

    /*Ŀ*/
    /* Write Agent's FONT to the PMSPY "profile"                            */
    /**/

    PrfWriteProfileString(hINI, szAgent, szKeyFont, pIni->szListFont);

    /*Ŀ*/
    /* Write Agent's WINDOW POSITION to the PMSPY "profile"                 */
    /**/

    if ( WinQueryWindowPos(hwndFrame, &pIni->swpAgent) )
    {
       sprintf(szData, "%d", pIni->swpAgent.x);
       PrfWriteProfileString(hINI, szAgent, szKeyPosX,   szData);

       sprintf(szData, "%d", pIni->swpAgent.y);
       PrfWriteProfileString(hINI, szAgent, szKeyPosY,   szData);

       sprintf(szData, "%d", pIni->swpAgent.cx);
       PrfWriteProfileString(hINI, szAgent, szKeySizeCX, szData);

       sprintf(szData, "%d", pIni->swpAgent.cy);
       PrfWriteProfileString(hINI, szAgent, szKeySizeCY, szData);
    }

    /*Ŀ*/
    /* Close the PMSPY "profile"                                            */
    /**/

    PrfCloseProfile(hINI);

    bOK = TRUE;
  }

  return( bOK );
}

/*Ŀ*/
/* Read Agent's PMSPY data from INI file (or set to default values)         */
/**/

static BOOL ReadSpyINI(HWND      hFrame,   /* HWND(Agent's frame) */
                       SPYEE     Agent,    /* one-origin "agent" */
                       PINI_DATA pIni)     /* INI data to read */
{
  CHAR szAgent[32];

  HINI hINI          = NULLH;
  BOOL bOK           = FALSE;

  /*Ŀ*/
  /* Set the INI data to 'default' values...                                */
  /*                                                                        */
  /*                                                                        */
  /* - list font                                                            */
  /*                                                                        */
  /* - window position:  x  = left edge                                     */
  /*                     y  = above bottom row of icons                     */
  /*                     cX = 6/10 screen width                             */
  /*                     cY = 4/10 screen height                            */
  /**/

  memset(pIni, 0, sizeof(*pIni) );

  strcpy(pIni->szListFont, szDefFont);

  pIni->swpAgent.x  = 0;

  pIni->swpAgent.y  = (SHORT)
                      (WinQuerySysValue(HWND_DESKTOP, SV_CYICON)      *2);

  pIni->swpAgent.cx = (SHORT)
                      (WinQuerySysValue(HWND_DESKTOP, SV_CXFULLSCREEN)*6)/10;

  pIni->swpAgent.cy = (SHORT)
                      (WinQuerySysValue(HWND_DESKTOP, SV_CYFULLSCREEN)*4)/10;

  /*Ŀ*/
  /* Open the PMSPY "profile"                                               */
  /**/

  if ( (hINI = PrfOpenProfile( WinQueryAnchorBlock(hFrame),
                               szINI)) != NULLH )
  {
    /*Ŀ*/
    /* Determine which "agent" this data is for                             */
    /**/

    sprintf(szAgent, Strings[IDS_FMT_AGENT], Agent);

    /*Ŀ*/
    /* Read Agent's previous FONT                                           */
    /**/

    PrfQueryProfileString(hINI,
                          szAgent,
                          szKeyFont,
                          pIni->szListFont,           /* default */
                          pIni->szListFont,           /* target & size */
                          (LONG)sizeof(pIni->szListFont) );

    /*Ŀ*/
    /* Read Agent's previos WINDOW POSITION                                 */
    /**/

    pIni->swpAgent.x  = PrfQueryProfileInt(hINI,
                                           szAgent,
                                           szKeyPosX,
                                           pIni->swpAgent.x);   /* default */

    pIni->swpAgent.y  = PrfQueryProfileInt(hINI,
                                           szAgent,
                                           szKeyPosY,

                                           pIni->swpAgent.y);   /* default */
    pIni->swpAgent.cx = PrfQueryProfileInt(hINI,
                                           szAgent,
                                           szKeySizeCX,
                                           pIni->swpAgent.cx);  /* default */

    pIni->swpAgent.cy = PrfQueryProfileInt(hINI,
                                           szAgent,
                                           szKeySizeCY,
                                           pIni->swpAgent.cy);  /* default */

    /*Ŀ*/
    /* Close the PMSPY "profile"                                            */
    /**/

    PrfCloseProfile(hINI);

    bOK = TRUE;
  }

  return( bOK );
}

/*Ŀ*/
/* Toggle menu 'check' on exclusive items                                   */
/**/
static VOID ToggleMenuChecking(register USHORT decheck, register USHORT check)
{
  WinSendMsg(hwndMenu, MM_SETITEMATTR,        /* de-check first */
             MPFROM2SHORT(decheck, TRUE),
             MPFROM2SHORT(MIA_CHECKED, 0) );

  WinSendMsg(hwndMenu, MM_SETITEMATTR,        /* then check */
             MPFROM2SHORT(check, TRUE),
             MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED) );
}

/*Ŀ*/
/* Check/Decheck a menu pulldown item                                       */
/**/
static VOID CheckMenuPulldown(register USHORT idItem,
                              register BOOL   bCheck)
{
  WinSendMsg(hwndMenu,                                     /* Menu window */
             MM_SETITEMATTR,                               /* Action */
             MPFROM2SHORT(idItem, TRUE),                   /* Go look for it! */
             MPFROM2SHORT(MIA_CHECKED,
                          bCheck ? MIA_CHECKED : 0) );     /* do right thing */
}

/*Ŀ*/
/* Enable/Disable menu pulldown item                                        */
/**/
static VOID EnableMenuPulldown(register USHORT idItem,
                               register BOOL bEnable)
{
  WinSendMsg(hwndMenu,                                     /* Menu window */
             MM_SETITEMATTR,                               /* Action */
             MPFROM2SHORT(idItem, TRUE),                   /* Go look for it! */
             MPFROM2SHORT(MIA_DISABLED,
                          bEnable ? 0 : MIA_DISABLED) );   /* do right thing */
}

/*Ŀ*/
/* Clear the Message List                                                   */
/**/
static VOID ClearMessageList( VOID )
{
  WinSendMsg(hwndMsgList, LM_DELETEALL, (MPARAM)NULL, (MPARAM)NULL);
  NbrOfMessages = 0;
}

/*Ŀ*/
/* Open/Append a Profile                                                    */
/**/

typedef enum {UseDefault, PromptForFile, UsePassedFile} PPF_MODE;

VOID ProcessProfileFile(HWND              hwndCur,
                        HWND              hwndFrame,
                        PSPY_DATA         pSpyData,
                        register BOOL     bOpen,  /* T=Open, F=Append */
                        register PPF_MODE mode,
                        PSZ               useName)

{
  register USHORT i;
           MSG    lowMsg, highMsg;
  FILEDLG  *FileDlg;

  /* Prompt user for PROFILE name IF we need to.... */

  switch( mode )
  {
    case PromptForFile:
         FileDlg           = calloc(sizeof(FILEDLG), 1);

         FileDlg->cbSize   = sizeof(FILEDLG);
         FileDlg->fl       = FDS_OPEN_DIALOG;
         FileDlg->pszTitle = Strings[IDS_FILE_ENTER_PROFILE];
         strcpy(FileDlg->szFullFile, szProPattern);

         WinFileDlg(HWND_DESKTOP, hwndCur, FileDlg);

         if (FileDlg->lReturn != DID_OK)
            {
            free(FileDlg);
            return;
            }

         strcpy(szFileName, FileDlg->szFullFile);
         free(FileDlg);
   break;

   default:    /* nothing to do.... */ ;
  }

  /*** reset SPY controls IFF open ***/

  if (bOpen)
    ResetSpyData(pSpyData);

  /*** Load the appropriate Profile ***/

  switch( mode )
  {
    case UsePassedFile:
         OpenProfile(hwndCur, hwndFrame, pSpyData, useName);
         break;

    case PromptForFile:
         OpenProfile(hwndCur, hwndFrame, pSpyData, szFileName);
         break;

    default:
         OpenProfile(hwndCur, hwndFrame, pSpyData, NULL);
         break;
  }

  /*** Pass new Message Range to DLL servant ***/

  if (pSpyData->TotalMsgs != 0)
  {
    /*Ŀ*/
    /* Process all of the MSG slots                                         */
    /**/
    for(/* Initialize */ i       = 0,         /* @ first MSG_SLOT */
                         lowMsg  = 0xFFFF,    /* outrageous LOW */
                         highMsg = 0;         /* outrageous HIGH */

        /* While      */ i < NO_MSG_SLOTS;    /* while not all examined */

        /* Iterate    */ i++)
    {
      /*Ŀ*/
      /* Check for range change IFF message(s) in this slot                 */
      /**/
      if (pSpyData->Msgs[i].uNoMsgs != 0)
      {
        if (pSpyData->Msgs[i].MsgLow < lowMsg)     /* new LOW range? */
          lowMsg = pSpyData->Msgs[i].MsgLow;

        if (pSpyData->Msgs[i].MsgHigh > highMsg)   /* new HIGH range? */
          highMsg = pSpyData->Msgs[i].MsgHigh;
      }
    }

    /*Ŀ*/
    /* Set new MSG range to optimize processing                             */
    /**/
    SpySetTargetMsgRange(SpyInstance, lowMsg, highMsg);
  }
}

/*Ŀ*/
/* Build array with the names of all Non-proportional FONT names            */
/*                                                                          */
/* Input:   hWnd.....HWND of window (for obtaining a temporary HPS)         */
/*                                                                          */
/* OutPut:  NULL.....unable to determine FONTS                              */
/*          !NULL....pointer to first CHAR szFacename[FACESIZE] element;    */
/*                   variable length list ended when szFacename[0] = NULL   */
/**/

PFONTNAME DetermineFixedFonts(HWND  hWnd)

{
   LONG             lReqFonts     = 0,          /* # FONTS requested */
                    lTotFonts     = 0;          /* # FONTS not processed */

   register USHORT   usI,
                     usFixed;

   HPS               hPS;

   PFONTMETRICS      pfmList        = NULL;
   PFONTNAME         pfnList        = NULL;

   FONTNAME          fnBuild;

   /*Ŀ*/
   /* First, we need a HPS before we can QUERY about the FONTS...           */
   /**/

   hPS = WinGetPS(hWnd);

   /*Ŀ*/
   /* Determine maximum number of FONTs possible                            */
   /**/

   lTotFonts = GpiQueryFonts(hPS,                 /* Handle(PS) */
                             QF_PUBLIC |          /* Enumerate all PUBLIC fonts */
                             QF_PRIVATE,          /*           and PRIVATE fonts */
                             NULL,                /* include all available fonts */
                             &lReqFonts,          /* use ZERO to get total */
                             (LONG)
                             sizeof(FONTMETRICS), /* sizeof(ONE font metrics) */
                             pfmList);            /* address(1st font metrics) */

   /*Ŀ*/
   /* Allocate array big enough for all those FONTs possible                */
   /**/


   pfmList = AllocateMemory( LOUSHORT(lTotFonts) * sizeof(FONTMETRICS) );

   /*Ŀ*/
   /* Enumerate all those FONTs possible                                    */
   /**/

   lReqFonts = lTotFonts;                     /* request all of them! */

   lTotFonts = GpiQueryFonts(hPS,             /* Handle(PS) */
                             QF_PUBLIC |      /* Enumerate all PUBLIC fonts */
                             QF_PRIVATE,      /*           and PRIVATE fonts */
                             NULL,            /* include all available fonts */
                             &lReqFonts,      /* Max # fonts requesting */
                             (LONG)
                             sizeof(FONTMETRICS), /* sizeof(ONE font metrics) */
                             pfmList);        /* address(font metrics) */

   WinReleasePS(hPS);

   /*Ŀ*/
   /* Now, determine how many are Fixed width FONTs                         */
   /**/

   for(usI     =                    /* Initialize: start @ first FONT */
       usFixed = 0;                 /*             no fixed FONTS, yet... */

       usI < LOUSHORT(lReqFonts);   /* While:      not all examined */
       usI++)                       /* Iterate:    move to the next FONT */
   {
     /*Ŀ*/
     /* Increment count if a Fixed width FONTs                              */
     /**/

     if (pfmList[usI].fsType  & FM_TYPE_FIXED)
       usFixed++;
   }

   /*Ŀ*/
   /* Allocate and build list with names of Fixed width FONTs               */
   /*                                                                       */
   /* - include an extra FONT name slot for the terminating NULL name...    */
   /**/

   pfnList = AllocateMemory( (usFixed + 1) * sizeof(FONTNAME) );

   for(usI     =                    /* Initialize: start @ first FONT */
       usFixed = 0;

       usI < LOUSHORT(lReqFonts);   /* While:      not all examined */
       usI++)                       /* Iterate:    move to the next FONT */
   {
     /*Ŀ*/
     /* Copy FONT name if a Fixed width FONTs                               */
     /**/

     if (pfmList[usI].fsType & FM_TYPE_FIXED)
     {
       PFONTNAME pFont;
       USHORT    i;

       sprintf(&fnBuild[0],
               "%d.%s",
               pfmList[usI].sNominalPointSize / 10,
               pfmList[usI].szFacename);

       /* test if this font is bold or italic */
       if (strstr(fnBuild, "Bold")) goto TryNextFont;
       if (strstr(fnBuild, "Italic")) goto TryNextFont;
       /* test if this font is already used */
       pFont = pfnList;                               /* @ first FONT */
       for (i=0; i<usFixed; i++) {
         if (strcmp(pFont[0], fnBuild) == 0) {
           /* already known font */
           goto TryNextFont;
         } /* endif */
         pFont++;
       } /* endfor */

       strcpy(pfnList[usFixed], fnBuild);

       usFixed++;

       TryNextFont: ;
     }
   }

   FreeMemory(pfmList);

   /*Ŀ*/
   /* Finally, return the results                                           */
   /**/

   return( pfnList );

}

/*Ŀ*/
/* Spy Window Procedure                                                     */
/**/

MRESULT SpyWindowProc(HWND   hwnd,
                      ULONG  msg,
                      MPARAM mp1,
                      MPARAM mp2)
{
  /*Ŀ*/
  /* Stack based AUTOMATIC variables                                        */
  /**/

  HPS                  hps;                    /* Presentation space handle   */

  RECTL                rectl;                  /* Window rectangle            */

  register USHORT      usWork;

  register PMSG_ITEM   pMsg;

  INT                  rc;

  CHAR                 szText[256],
                       szFormat[64];

  HWND                 hwndTarget;

  HMQ                  hmqTarget;

  POINTL               ptrPos;

  MRESULT              wpResult = NULL;

  FILEDLG              *FileDlg;
  FILEFINDBUF3         FileFindBuf;      /* for DosFindFirst/Next             */
  HDIR                 hdir;             /* for DosFindFirst/Next             */
  USHORT               attrib;           /* for DosFindFirst/Next             */
  ULONG                count;            /* for DosFindFirst/Next             */


  /*Ŀ*/
  /* STATIC variables preserved across each MSG                             */
  /**/

  static BOOL          bAutoScrollList             = TRUE;
  static BOOL          bWindowIsObject             = FALSE;
  static BOOL          bIncludeUndefinedWM_USER    = FALSE;
  static BOOL          bIncludeUndefinedMsgs       = FALSE;
  static BOOL          bSelecting                  = FALSE;

  static USHORT        MaxNbrOfMessages      = 0;  /* dynamically set from .RC */
  static USHORT        MaxDeleteItems        = 1;  /* dynamically set from .RC */

  static HPOINTER      hSpy                  = NULLH,
                       hOld                  = NULLH;

  static PFONTNAME     pFontList             = NULL;

  /*Ŀ*/
  /* Process this MSG                                                       */
  /**/

  switch(msg)
  {
    /*Ŀ*/
    /* Message from one of our "hook" procedures                            */
    /*                                                                      */
    /* Mp1............PQMSG to data for "hooked" MSG                        */
    /* Mp2............LOUSHORT = HK_* ID of message source                  */
    /*                HIUSHORT = Not Used                                   */
    /*                                                                      */
    /* - decide if this instance should INCLUDE this MSG                    */
    /* - if so: a) format message                                           */
    /*          b) add to list                                              */
    /*          c) LOG if logging active                                    */
    /*                                                                      */
    /* Notes:                                                               */
    /* ------                                                               */
    /* - since this is the most time critical MSG, it should be left as     */
    /*   the FIRST case of the switch statement!                            */
    /**/

    case PMSPY_MSG_FROM_HOOK:
    {
      PQMSG     pSpyMsg = (PQMSG) mp1;  /* locate passed "hooked" data item */

      MSG_COLOR ClrFG;                  /* Msg color */

      PSZ       pszHK   = LOUSHORT(mp2) == HK_INPUT  /* determine is Sent/Posted */
                          ? Strings[IDS_MSG_POSTED]
                          : Strings[IDS_MSG_SENT];


      /*Ŀ*/
      /* Determine if MSG wanted                                            */
      /*                                                                    */
      /* NO if: 1) the message displaying is 'frozen' at the moment         */
      /*        2) the message isn't found in the current profile           */
      /**/
      szText[0] = 0;                      /* indicate "no msg formatted" */

      if ((pMsg = FindMsg(pSpyData, pSpyMsg->msg)) != NULLH) /* locate MSG data? */
      {
        if (pMsg->TriggerThaw && slmState==SLM_FROZEN) {
          slmState = SLM_THAWED;
          WinSendMsg(hwnd,
                     PMSPY_ADD_TO_LIST,
                     MPFROMP(Strings[IDS_STAT_TRIGGERED_THAW]),
                     MPFROM2SHORT(Color_Status, FALSE) );
          ClrFG = Color_Status;
          goto AddMessageToList;
        } else if (pMsg->TriggerFreeze && slmState==SLM_THAWED) {
          slmState = SLM_FROZEN;
          WinSendMsg(hwnd,
                     PMSPY_ADD_TO_LIST,
                     MPFROMP(Strings[IDS_STAT_TRIGGERED_FREEZE]),
                     MPFROM2SHORT(Color_Status, FALSE) );
          ClrFG = Color_Status;
          goto AddMessageToList;
        } /* endif */

        if ( slmState == SLM_FROZEN )          /* MSG display (possibly) active? */
          return( NULL );

        if ( pMsg->Include )
        {
           /*Ŀ*/
           /* This message is: (1) Defined                                  */
           /*                  (2) to be included                           */
           /**/

           ClrFG = pMsg->ClrFG;        /* using real Msg's color */

           AddMessageToList:
           sprintf(szText,
                   Strings[IDS_FMT_STANDARD],
                   pMsg->pDesc,
                   pSpyMsg->mp1,
                   pSpyMsg->mp2,
                   pszHK,
                   pSpyMsg->hwnd);
        }
        /*Ŀ*/
        /* This message is: (1) Defined                                     */
        /*                  (2) NOT to be included                          */
        /**/
        else
        {
          return( NULL );
        }
      }
      /*Ŀ*/
      /* This message is: (1) NOT Defined                                   */
      /**/
      else
      {

        if ( pSpyMsg->msg >= WM_USER ) /* WM_USER message? */
        {
          if ( !bIncludeUndefinedWM_USER )
            return( NULL );

          ClrFG = Color_UndefinedWM_USER;

          sprintf(szFormat,
                  Strings[IDS_FMT_USER_MSG],
                  pSpyMsg->msg );

          sprintf(szText,
                  Strings[IDS_FMT_STANDARD],
                  szFormat,
                  pSpyMsg->mp1,
                  pSpyMsg->mp2,
                  pszHK,
                  pSpyMsg->hwnd);
        }
        else
        {
          if ( !bIncludeUndefinedMsgs )
            return( NULL );

          ClrFG = Color_UndefinedMsg;

          sprintf(szFormat,
                  Strings[IDS_FMT_UNKNOWN_MSG],
                  pSpyMsg->msg );

          sprintf(szText,
                  Strings[IDS_FMT_STANDARD],
                  szFormat,
                  pSpyMsg->mp1,
                  pSpyMsg->mp2,
                  pszHK,
                  pSpyMsg->hwnd);
        }
      }
      /*Ŀ*/
      /* Message to add to the "Spy list"?                                  */
      /**/
      if (szText[0] != 0)              /* did we format a msg? */
      {
        /*Ŀ*/
        /* add the message                                                  */
        /**/
        WinSendMsg(hwnd,
                   PMSPY_ADD_TO_LIST,
                   MPFROMP(szText),
                   MPFROM2SHORT(ClrFG, FALSE) );
      }
    }
    break;

    /*Ŀ*/
    /* Add a text Message to the SPY list                                   */
    /*                                                                      */
    /* Mp1............PSZ to message text                                   */
    /* Mp2............LOUSHORT = binary Color index                         */
    /*                HIUSHORT = TRUE.....draw in "Half tone"               */
    /*                           FALSE....draw in regular tone              */
    /*                                                                      */
    /* a) if list is full, make room                                        */
    /* b) add to list                                                       */
    /* c) LOG if logging active                                             */
    /* d) free string if told to                                            */
    /**/

    case PMSPY_ADD_TO_LIST:
    {
      HOOK_DATA  AddData;

      /*Ŀ*/
      /* is the SPY list now full?                                          */
      /**/

      if (++NbrOfMessages == MaxNbrOfMessages)
      {
        NbrOfMessages -= MaxDeleteItems;      /* Yes...delete group of "oldest" */

        for (usWork = 0;                      /* Init: */
             usWork < MaxDeleteItems;         /* While: limit not deleted */
             usWork++)                        /* Iter:  increment count */
        {
          WinSendMsg(hwndMsgList,             /* delete current "Top" */
                     LM_DELETEITEM,           /* (which is 'oldest') */
                     MPFROMSHORT(0),
                     NULL);
        } /* endfor */
      }

      /*Ŀ*/
      /* Final formatting of message before adding to list                  */
      /**/

      AddData.logCLR      = (CHAR)(LOUSHORT(mp2) + COLOR_BASE);
      AddData.useHalfTone = (CHAR)( (HIUSHORT(mp2) == TRUE) ? '1' : '0');
      strcpy(AddData.szMsg, mp1);

      /*Ŀ*/
      /* insert the message at the end of the list (since time ordered)     */
      /**/

      WinSendMsg(hwndMsgList,
                 LM_INSERTITEM,
                 MPFROMSHORT(LIT_END),
                 MPFROMP(&AddData));

      /*Ŀ*/
      /* Make added item visible (by forcing it to the top of list)         */
      /**/

      if (bAutoScrollList)
        WinSendMsg(hwndMsgList,
                   LM_SETTOPINDEX,
                   MPFROMSHORT(max(0, NbrOfMessages - sNbrOfItems)),
                   NULL);

      /*Ŀ*/
      /* are we logging to a file?                                          */
      /**/
      if (bCurrentlyLogging)
        fprintf(pLogFile, "%s\n", AddData.szMsg);

      /*Ŀ*/
      /* free message text if so instructed...                              */
      /**/
      if ( HIUSHORT(mp2) )
        FreeMemory( mp1 );
    }
    break;

    /*Ŀ*/
    /* Provide size of item to be drawn in the SPY message list             */
    /*                                                                      */
    /* - this is part of the responsibility of an OWNERDRAW listbox         */
    /**/

    case WM_MEASUREITEM:
    {
         FONTMETRICS Metrics;
         hps = WinGetPS(hwnd);
         GpiQueryFontMetrics(hps, (LONG)sizeof(FONTMETRICS), &Metrics );
         WinReleasePS(hps);
         lItemHeight = Metrics.lMaxBaselineExt;
         wpResult = MRFROM2SHORT( LOUSHORT(Metrics.lMaxBaselineExt),
                                  L_NOTES_TEXT * LOUSHORT(Metrics.lAveCharWidth));         /* item WIDTH */
    }
    break;

    /*Ŀ*/
    /* Draw the specified item in the SPY message list                      */
    /*                                                                      */
    /* - this is part of the responsibility of an OWNERDRAW listbox         */
    /* - the ITEM text is really the data encoded HOOK_DATA structure       */
    /*   built of ASCII characters to 'survive' being added to the listbox  */
    /**/

    case WM_DRAWITEM:
    {
      HOOK_DATA  DrawData;

      POWNERITEM pOwn = (POWNERITEM)mp2;

      USHORT     usFlags = DT_LEFT     |      /* Horizontally left justified */
                           DT_VCENTER  |      /* Vertically   centered */
                           DT_ERASERECT;      /* auto ERASE rectangle, Thanks! */

      if ( (pOwn->fsState    == pOwn->fsStateOld) || /* not a selection change */
           (pOwn->fsStateOld && pOwn->fsState) )
      {
        /* Get text of item to DRAW */

        WinSendMsg(pOwn->hwnd,
                   LM_QUERYITEMTEXT,
                   MPFROM2SHORT(pOwn->idItem, sizeof(DrawData)),
                   (MPARAM)&DrawData);

        /* Determine if "half tone" drawing is required... */

        if ( DrawData.useHalfTone == '1' )
          usFlags |= DT_HALFTONE;

        /* Draw using specified COLOR */

        WinDrawText(pOwn->hps,          /* target HPS */
                    -1,                 /* using PSZ string, let PM calculate */
                    DrawData.szMsg,     /* item text */
                    &pOwn->rclItem,     /* target list item rectangle */
ExternalColorTranslation[pSpyData->LogicalColorTranslation[DrawData.logCLR-COLOR_BASE].iExtColor].lClrValue,
                    SYSCLR_ENTRYFIELD,  /* BG color for ListBox */
                    usFlags);
      }
      wpResult = MRFROMSHORT(TRUE);    /* Keep PM from drawing it too */
    }
    break;

    /*Ŀ*/
    /* Frame has been re-sized...adjust client area size to fit within it   */
    /**/
    case WM_SIZE:

      WinQueryWindowRect(hwnd,&rectl);
      WinSetWindowPos(hwndMsgList,
                      HWND_TOP,
                      0,0,
                      (SHORT)rectl.xRight,
                      (SHORT)rectl.yTop,
                      SWP_SIZE | SWP_MOVE);
      sNbrOfItems = (SHORT)(rectl.yTop / max(1L, lItemHeight));
      break;

    /*Ŀ*/
    /* the user is trying to CLOSE us...                                    */
    /**/

    case WM_CLOSE:

         SpyUnSetTarget(SpyInstance);

         WinSendMsg(hwnd,
                    PMSPY_ADD_TO_LIST,
                    MPFROMP(Strings[IDS_STAT_CLOSE]),
                    MPFROM2SHORT(Color_Status, FALSE) );

         /* close logfile if still open */

         if (pLogFile)
           fclose(pLogFile);

         /* tell main window to terminate */

         wpResult = WinDefWindowProc(hwnd, msg, mp1, mp2);

    break;

    /*Ŀ*/
    /* This message is sent each time a PULLDOWN menu is to be used         */
    /*                                                                      */
    /* - we dynamically enable/disable various items based on our 'state'   */
    /**/

    case WM_INITMENU:
    {
      switch (LOUSHORT(mp1))
      {
        case ID_AB_SPYON:
          if (SpyQueryTargetWindow(SpyInstance) == SPY_BAD_HWND )
          {
            ToggleMenuChecking(ID_SELECT,   ID_DESELECT);  /* nothing SPYed */
            EnableMenuPulldown(ID_SELECT,   TRUE);
            EnableMenuPulldown(ID_DESELECT, FALSE);
          }
          else
          {
            ToggleMenuChecking(ID_DESELECT, ID_SELECT);
            EnableMenuPulldown(ID_SELECT,   TRUE);
            EnableMenuPulldown(ID_DESELECT, TRUE);
          }
        break;

        case ID_AB_SPYWHAT:
          if ( SpyQueryTargetIsWindow(SpyInstance)  )
            ToggleMenuChecking(IDD_QUE, IDD_WND);
          else
            ToggleMenuChecking(IDD_WND, IDD_QUE);
        break;

        case ID_AB_FILE:

          ToggleMenuChecking(pLogFile == NULL ? ID_F_OPENLOG   : ID_F_CLOSELOG,
                             pLogFile == NULL ? ID_F_CLOSELOG  : ID_F_OPENLOG);

          EnableMenuPulldown(ID_F_OPENLOG,  pLogFile == NULL);
          EnableMenuPulldown(ID_F_CLOSELOG, pLogFile != NULL);

        break;

        case ID_AB_OPTIONS:

          /* decide which of the Freeze/Thaw options is active now */

          CheckMenuPulldown(ID_O_THAW,
                            slmState == SLM_THAWED           ? TRUE : FALSE);
          CheckMenuPulldown(ID_O_FREEZE,
                            slmState == SLM_FROZEN           ? TRUE : FALSE);

          if (pLogFile != NULL) /* LOGing active? */
          {
            ToggleMenuChecking(bCurrentlyLogging ? ID_O_STOPLOG  : ID_O_STARTLOG,
                               bCurrentlyLogging ? ID_O_STARTLOG : ID_O_STOPLOG);
            EnableMenuPulldown(ID_O_STARTLOG, !bCurrentlyLogging);
            EnableMenuPulldown(ID_O_STOPLOG,   bCurrentlyLogging);
          }
          else
          {
            EnableMenuPulldown(ID_O_STARTLOG, FALSE);
            EnableMenuPulldown(ID_O_STOPLOG,  FALSE);
          }
          CheckMenuPulldown(ID_O_SCROLL,     bAutoScrollList );
        break;

        case ID_AB_FILTER:
          CheckMenuPulldown(ID_F_INC_WM_USER,   bIncludeUndefinedWM_USER );
          CheckMenuPulldown(ID_F_INC_UNDEFINED, bIncludeUndefinedMsgs );
        break;
      } /* endswitch */
    }
    break;

    /*Ŀ*/
    /* This message is sent each time a PULLDOWN menu is selected           */
    /**/

    case WM_COMMAND:
      switch (SHORT1FROMMP(mp1))
      {
        case ID_F_EXIT:
          WinPostMsg(hwnd, WM_CLOSE, NULL, NULL);
        break;

        case ID_H_ABOUT:
          WinDlgBox(HWND_DESKTOP,
                    hwnd,
                    (PFNWP)SpyLogoDlgProc,
                    hmodNLS,
                    IDD_ABOUT,
                    NULL);
        break;

        /*Ŀ*/
        /* User want to "point and shoot" at the window to be SPYed...      */
        /*                                                                  */
        /* - WinSetCapture will route all MOUSE messages to this window     */
        /* - The MOUSE messages are processing below                        */
        /**/

        case ID_SELECT:

          bSelecting = TRUE;

          SpyUnSetTarget(SpyInstance);

/*        ClearMessageList();            **** preserve NOTES in list ****/

          WinSetCapture(HWND_DESKTOP, hwnd);

          hOld = WinQueryPointer(HWND_DESKTOP);
          WinSetPointer(HWND_DESKTOP, hSpy);

          /* force the window in the background */

          WinSetWindowPos(hwndFrame, HWND_BOTTOM, 0, 0, 0, 0, SWP_ZORDER);
        break;

        /*Ŀ*/
        /* User want to select an OBJECT window for SPYing...               */
        /**/
        case ID_SELECT_OBJECT:
        {
           static OW_DATA OWData =
           {
             NULLH,                     /* hwndNotify:   dynamically filled */
             PMSPY_OW_RESULT,           /* uNotifyMsg:   dialog result MSG */

             FALSE,                     /* fUseData:     use dialog data? */

             NULLH,                     /* hwndOW:       initially NULLH */
             NULLH                      /* hmqOW:        initially NULLH */
           };

           OWData.hwndNotify = hwnd;    /* return result to ME! */

              WinDlgBox( HWND_DESKTOP,
                         hwnd,
                         ObjectDlgProc,
                         hmodNLS,
                         ID_OW_DLG,
                         &OWData);
        }
        break;

        case ID_DESELECT:
          SpyUnSetTarget(SpyInstance);
          bWindowIsObject = FALSE;
          UpdateSpyTitle(hwndFrame, tStandard);
          WinSendMsg(hwnd,
                     PMSPY_ADD_TO_LIST,
                     MPFROMP(Strings[IDS_STAT_DESELECTED]),
                     MPFROM2SHORT(Color_Status, FALSE) );
        break;

        case ID_O_CLEAR:
          ClearMessageList();
        break;

        /*Ŀ*/
        /* "Thaw" the message list (enable message additions)               */
        /**/
        case ID_O_THAW:
          slmState = SLM_THAWED;
          WinSendMsg(hwnd,
                     PMSPY_ADD_TO_LIST,
                     MPFROMP(Strings[IDS_STAT_LIST_ON]),
                     MPFROM2SHORT(Color_Status, FALSE) );
        break;

        /*Ŀ*/
        /* "Freeze" the message list (disable message additions)            */
        /**/
        case ID_O_FREEZE:
          slmState = SLM_FROZEN;
          WinSendMsg(hwnd,
                     PMSPY_ADD_TO_LIST,
                     MPFROMP(Strings[IDS_STAT_LIST_OFF]),
                     MPFROM2SHORT(Color_Status, FALSE) );
        break;

        case ID_F_CLOSELOG:
          WinSendMsg(hwnd, WM_COMMAND, MPFROMSHORT(ID_O_STOPLOG), NULL);
          fclose(pLogFile);
          pLogFile = (FILE *)NULL;
        break;

        case ID_F_OPENLOG:

          FileDlg           = calloc(sizeof(FILEDLG), 1);

          FileDlg->cbSize   = sizeof(FILEDLG);
          FileDlg->fl       = FDS_SAVEAS_DIALOG | FDS_ENABLEFILELB;
          strcpy(FileDlg->szFullFile, szLogPattern);

          WinFileDlg(HWND_DESKTOP, hwnd, FileDlg);

          if (FileDlg->lReturn != DID_OK)
             {
             free(FileDlg);
             break;
             }

          strcpy(szFileName, FileDlg->szFullFile);
          free(FileDlg);

          hdir   = HDIR_CREATE;
          attrib = FILE_NORMAL | FILE_READONLY | FILE_HIDDEN | FILE_SYSTEM | FILE_ARCHIVED;
          count  = 1;

          rc = DosFindFirst(szFileName,
                            &hdir,
                            attrib,
                            &FileFindBuf,
                            sizeof(FileFindBuf),
                            &count,
                            FIL_STANDARD);
          if (rc == 0 && count == 1)
             {
             rc = 0;
             }
          else
             {
             rc = 1;
             }

          /* if file exists but they don't want to replace, quit */
          if ( (rc == 0) &&
               (MsgDisplay(hwnd,
                           swcntrl.szSwtitle,
                           Strings[IDS_MSG_LOG_EXISTS],
                           IDS_MSG_LOG_EXISTS,
                           MB_ICONQUESTION | MB_YESNO | MB_MOVEABLE,
                           szFileName ) != MBID_YES) )
            break;

          pLogFile = fopen(szFileName, "w");

          /**** N O T E:  falling through......  ****/

        case ID_O_STARTLOG:
          bCurrentlyLogging = TRUE;

          if (SpyQueryTargetWindow(SpyInstance) != SPY_BAD_HWND )
          {
            WinSendMsg(hwnd,
                       PMSPY_ADD_TO_LIST,
                       MPFROMP(Strings[IDS_STAT_LOG_ON]),
                       MPFROM2SHORT(Color_Status, FALSE) );

            if (SpyQueryTargetIsWindow(SpyInstance))
              fprintf(pLogFile,
                      Strings[IDS_FMT_SPY_WINDOW],
                      SpyQueryTargetWindow(SpyInstance));
            else
              fprintf(pLogFile,
                      Strings[IDS_FMT_SPY_QUEUE],
                      SpyQueryTargetQueue(SpyInstance));

            fputc('\n', pLogFile);
          }
        break;

        case ID_O_STOPLOG:
          bCurrentlyLogging = FALSE;
          WinSendMsg(hwnd,
                     PMSPY_ADD_TO_LIST,
                     MPFROMP(Strings[IDS_STAT_LOG_OFF]),
                     MPFROM2SHORT(Color_Status, FALSE) );
        break;

        case ID_F_APPENDPROFILE:
             ProcessProfileFile(hwnd,
                                hwndFrame,
                                pSpyData,
                                FALSE,
                                PromptForFile,
                                NULL);
        break;

        case ID_F_OPENPROFILE:
             ProcessProfileFile(hwnd,
                                hwndFrame,
                                pSpyData,
                                TRUE,
                                PromptForFile,
                                NULL);
        break;

        case ID_F_SAVEPROFILE:
          FileDlg           = calloc(sizeof(FILEDLG), 1);

          FileDlg->cbSize   = sizeof(FILEDLG);
          FileDlg->fl       = FDS_SAVEAS_DIALOG | FDS_ENABLEFILELB;
          FileDlg->pszTitle = Strings[IDS_FILE_ENTER_PROFILE];
          strcpy(FileDlg->szFullFile, szProPattern);

          WinFileDlg(HWND_DESKTOP, hwnd, FileDlg);

          if (FileDlg->lReturn != DID_OK)
             {
             free(FileDlg);
             break;
             }

          hdir   = HDIR_CREATE;
          attrib = FILE_NORMAL | FILE_READONLY | FILE_HIDDEN | FILE_SYSTEM | FILE_ARCHIVED;
          count  = 1;

          rc = DosFindFirst(szFileName,
                            &hdir,
                            attrib,
                            &FileFindBuf,
                            sizeof(FileFindBuf),
                            &count,
                            FIL_STANDARD);
          if (rc == 0 && count == 1)
             {
             rc = 0;
             }
          else
             {
             rc = 1;
             }

          strcpy(szFileName, FileDlg->szFullFile);
          free(FileDlg);

          if ( (rc == 0) &&
               (MsgDisplay(hwnd,
                           swcntrl.szSwtitle,
                           Strings[IDS_MSG_PROFILE_EXISTS],
                           IDS_MSG_PROFILE_EXISTS,
                           MB_ICONQUESTION | MB_YESNO | MB_MOVEABLE,
                           szFileName ) != MBID_YES) )
            break;
          SaveProfile(hwndFrame, pSpyData, szFileName);
        break;

        case ID_F_OPENDEFAULT:
            ProcessProfileFile(hwnd, hwndFrame, pSpyData, TRUE,  UseDefault, NULL);
        break;

        case ID_F_APPENDDEFAULT:
            ProcessProfileFile(hwnd, hwndFrame, pSpyData, FALSE, UseDefault, NULL);
        break;

        case ID_O_BROWSEDEFAULT:
             /* display Default PROFILE panel */
             WinSendMsg(hwndIPF,                    /* hwnd: IPF */
                        HM_DISPLAY_HELP,            /* msg:  display panel */
                        MPFROMSHORT(PID_DEFAULT_PROFILE), /* mp1:  which panel */
                        MPFROMSHORT(HM_RESOURCEID)); /* mp2: mp1 is ID */

        break;

        case ID_F_INCLUDE:
          IncludeAllMsgs( pSpyData, TRUE,  Color_Asis );
        break;

        case ID_F_EXCLUDE:
          IncludeAllMsgs( pSpyData, FALSE, Color_Asis );
        break;

        case ID_F_SELECTIVE:
           WinDlgBox( HWND_DESKTOP,
                      hwnd,
                      (PFNWP)FilterDlgProc,
                      hmodNLS,
                      IDD_FILTER,
                      pSpyData);
        break;

        case ID_F_GROUP:
           WinDlgBox( HWND_DESKTOP,
                      hwnd,
                      (PFNWP)GroupDlgProc,
                      hmodNLS,
                      ID_GRP_DLG,
                      pSpyData);
        break;

        case IDD_WND:
          SpySetTargetIsWindow(SpyInstance, TRUE);

          if ( SpyQueryTargetWindow(SpyInstance) != SPY_BAD_HWND)
            UpdateSpyTitle(hwndFrame,
                           bWindowIsObject ? tObject : tWindow,
                           SpyQueryTargetWindow(SpyInstance));
        break;

        case IDD_QUE:
          SpySetTargetIsWindow(SpyInstance, FALSE);

          if ( SpyQueryTargetQueue(SpyInstance) != SPY_BAD_HMQ)
            UpdateSpyTitle(hwndFrame,
                           tQueue,
                           SpyQueryTargetQueue(SpyInstance));
        break;

        /*Ŀ*/
        /* toggle "Include undefined WM_USER msg" flag                      */
        /**/

        case ID_F_INC_WM_USER:
             bIncludeUndefinedWM_USER ^= TRUE;  /* flip-flop current value */
        break;

        /*Ŀ*/
        /* toggle "Include undefined msg" flag                              */
        /**/

        case ID_F_INC_UNDEFINED:
             bIncludeUndefinedMsgs ^= TRUE;  /* flip-flop current value */
        break;

        /*Ŀ*/
        /* toggle "Auto scroll list" flag                                   */
        /**/

        case ID_O_SCROLL:
             bAutoScrollList ^= TRUE;  /* flip-flop current value */
        break;

        /*Ŀ*/
        /* Notes dialog                                                     */
        /**/

        case ID_O_NOTES:
        {
           static NOTES_DATA NotesData =
           {
             NULLH,                     /* hwndNotify:   dynamically filled */
             PMSPY_NOTES_RESULT,        /* uNotifyMsg:   dialog result MSG */

             FALSE,                     /* fUseData:     use dialog data? */

             ""                         /* szText:       initially NULL */
           };

           NotesData.hwndNotify = hwnd; /* return result to ME! */

           WinDlgBox( HWND_DESKTOP,
                      hwnd,
                      (PFNWP)NotesDlgProc,
                      hmodNLS,
                      DLG_NOTES,
                      &NotesData);
        }
        break;

        /*Ŀ*/
        /* Save the options data                                            */
        /**/
        case ID_O_SAVEOPTS:
          WinSendMsg(hwnd, WM_SAVEAPPLICATION, NULL, NULL);
        break;

        /*Ŀ*/
        /* FONT dialog                                                      */
        /**/

        case ID_O_FONT:
        {
           static FONT_DATA FontData =
           {
             NULLH,                     /* hwndNotify:   dynamically filled */
             PMSPY_FONT_RESULT,         /* uNotifyMsg:   dialog result MSG */

             FALSE                      /* fUseData:     use dialog data? */
           };

           FontData.hwndNotify = hwnd;                 /* return result to ME! */
           FontData.pfnList    = pFontList;            /* pass list of FONTS */
           FontData.pfnCurrent = (PFONTNAME)iniAgent.szListFont;  /* current FONT */

           WinDlgBox( HWND_DESKTOP,
                      hwnd,
                      (PFNWP)FontDlgProc,
                      hmodNLS,
                      DLG_FONT,
                      &FontData);
        }
        break;

        /*Ŀ*/
        /* Pulldown we don't care about...but check if some IPF work to do! */
        /**/
        default:
            HandleIPF(hwnd, msg, mp1, mp2, &wpResult);

      } /* endswitch for WM_COMMAND */
    break;

    /*Ŀ*/
    /* User pressed mouse button #1..                                       */
    /*                                                                      */
    /* - if they are 'selecting' a window, determine which window is now    */
    /*   under the POINTER                                                  */
    /**/

    case WM_BUTTON1DOWN: /* select for spying */

      if (bSelecting)
      {
        bSelecting = FALSE;

        WinSetPointer(HWND_DESKTOP, hOld);
        WinSetCapture(HWND_DESKTOP, NULLH);

        ptrPos.x = (LONG)SHORT1FROMMP(mp1);
        ptrPos.y = (LONG)SHORT2FROMMP(mp1);

        WinMapWindowPoints(hwnd, HWND_DESKTOP, (PPOINTL)&ptrPos, 1);
        hwndTarget = WinWindowFromPoint(HWND_DESKTOP, (PPOINTL)&ptrPos,
                                      TRUE);
        hmqTarget  = (HMQ)WinQueryWindowULong(hwndTarget, QWL_HMQ);

        /* trying to "spy" on the SPY? */

        if (hmqTarget == (HMQ)WinQueryWindowULong(hwnd, QWL_HMQ) )
        {
          MsgDisplay( hwnd,
                      swcntrl.szSwtitle,
                      Strings[IDS_FMT_SPY_THE_SPY],
                      IDS_FMT_SPY_THE_SPY,
                      MB_CUAWARNING | MB_CANCEL );
          UpdateSpyTitle(hwndFrame, tStandard); /* reset Window Title */
        }
        else {
          SpySetTarget(SpyInstance, hwndTarget, hmqTarget);
          bWindowIsObject = FALSE;

          UpdateSpyTitle(hwndFrame,
                         SpyQueryTargetIsWindow(SpyInstance)
                         ? tWindow
                         : tQueue,
                         SpyQueryTargetIsWindow(SpyInstance)
                         ? hwndTarget
                         :  hmqTarget);
        } /* endif */
      }
      else
        wpResult = WinDefWindowProc(hwnd, msg, mp1, mp2);
    break;

    /*Ŀ*/
    /* User pressed mouse button #2..                                       */
    /*                                                                      */
    /* - if they are 'selecting' a window, display the window class, handle,*/
    /*   and queue that is under the POINTER                                */
    /**/

    case WM_BUTTON2DOWN:  /* display hwnd and hmq */

      if (bSelecting)
      {
        ptrPos.x = (LONG)SHORT1FROMMP(mp1);
        ptrPos.y = (LONG)SHORT2FROMMP(mp1);

        WinMapWindowPoints(hwnd, HWND_DESKTOP, (PPOINTL)&ptrPos, 1);

        hwndTarget = WinWindowFromPoint(HWND_DESKTOP,
                                        (PPOINTL)&ptrPos, TRUE);
         hmqTarget = (HMQ)WinQueryWindowULong(hwndTarget, QWL_HMQ);

        WinQueryClassName(hwndTarget, sizeof(szText), szText);

        UpdateSpyTitle(hwndFrame,
                       tSpyWho,
                       TranslateClassName(szText), hwndTarget, hmqTarget);
      }
      else
        wpResult = WinDefWindowProc(hwnd, msg, mp1, mp2);
    break;

    /*Ŀ*/
    /* User double clicked button #2...                                     */
    /*                                                                      */
    /* - if they are 'selecting' a window, cancel operation                 */
    /**/

    case WM_BUTTON2DBLCLK:

      if (bSelecting)
      {
        bSelecting = FALSE;

        WinSetPointer(HWND_DESKTOP, hOld);
        WinSetCapture(HWND_DESKTOP, NULLH);

        UpdateSpyTitle(hwndFrame, tStandard);
      }
      else
        wpResult = WinDefWindowProc(hwnd, msg, mp1, mp2);
    break;

    /*Ŀ*/
    /* The mouse has moved....                                              */
    /*                                                                      */
    /* - if they are 'selecting' a window insure that our POINTER is used   */
    /**/

    case WM_MOUSEMOVE:

      if (bSelecting)
        WinSetPointer(HWND_DESKTOP, hSpy);
      else
        wpResult = WinDefWindowProc(hwnd, msg, mp1, mp2);
    break;

    /*Ŀ*/
    /* Re-Paint the client area                                             */
    /**/

    case WM_PAINT:

         hps = WinBeginPaint( hwnd, NULLH, &rectl );

         WinFillRect(hps, &rectl, SYSCLR_WINDOW );

         WinEndPaint( hps );
    break;

    /*Ŀ*/
    /* Notes dialog result                                                  */
    /*                                                                      */
    /* Mp1: PNOTES_DATA                                                     */
    /* Mp2: NULL (not used)                                                 */
    /**/

    case PMSPY_NOTES_RESULT:
    {
       PNOTES_DATA pNotes = (PNOTES_DATA) mp1;

       if (pNotes->fUseData)
       {
         WinSendMsg(hwnd,
                    PMSPY_ADD_TO_LIST,
                    MPFROMP(pNotes->szText),
                    MPFROM2SHORT(Color_Notes, FALSE) );
       }
    }
    break;

    /*Ŀ*/
    /* FONT dialog result                                                   */
    /*                                                                      */
    /* Mp1: PFONT_DATA                                                      */
    /* Mp2: NULL (not used)                                                 */
    /**/

    case PMSPY_FONT_RESULT:
    {
       PFONT_DATA pData = (PFONT_DATA) mp1;

       if (pData->fUseData)
       {
         strcpy( (PVOID)iniAgent.szListFont, (PVOID)pData->pfnSelected);

         WinSetPresParam(hwndMsgList,
                         PP_FONTNAMESIZE,
                         (ULONG)
                         strlen((PSZ)iniAgent.szListFont)+1,  /* must include NULL! */
                         iniAgent.szListFont);
       }
    }
    break;

    /*Ŀ*/
    /* OBJECT Window dialog result                                          */
    /*                                                                      */
    /* Mp1: POW_DATA                                                        */
    /* Mp2: NULL (not used)                                                 */
    /**/

    case PMSPY_OW_RESULT:
    {
       POW_DATA pOW = (POW_DATA) mp1;

       if (pOW->fUseData)
       {
          /*Ŀ*/
          /* Switch to SPYing this OBJECT window...                         */
          /*                                                                */
          /* - clear old SPYee                                              */
          /* - set   new SPYee                                              */
          /* - update title                                                 */
          /**/

          SpyUnSetTarget(SpyInstance);

          SpySetTarget(SpyInstance, pOW->hwndOW, pOW->hmqOW);

          bWindowIsObject = TRUE;

          UpdateSpyTitle(hwndFrame,
                         SpyQueryTargetIsWindow(SpyInstance)
                         ? tObject
                         : tQueue,
                         SpyQueryTargetIsWindow(SpyInstance)
                         ? pOW->hwndOW
                         : pOW->hmqOW);
       }
    }
    break;

    /*Ŀ*/
    /* Save current PMSPY setting for this "agent"                          */
    /**/

    case WM_SAVEAPPLICATION:

         WriteSpyINI( hwndFrame,          /* agent's frame HWND */
                      SpyInstance + 1,    /* one-origin "agent" */
                      &iniAgent);         /* agent's data */
    break;

    /*Ŀ*/
    /* Special one-time initialization processing                           */
    /**/

    case WM_CREATE:
    {
      /*Ŀ*/
      /* Determine all possible Fixed FONTs....                             */
      /**/

      pFontList = DetermineFixedFonts( hwnd );

      /*Ŀ*/
      /* Initialize SPY data for Processing                                 */
      /**/

      InitializeSpyData(pSpyData);

      /*Ŀ*/
      /* Load the 'default' PROFILE                                         */
      /**/

      ProcessProfileFile(hwnd, hwndFrame, pSpyData, TRUE, UseDefault, NULL);

      /*Ŀ*/
      /* Load our special POINTER for selecting windows                     */
      /**/

      hSpy = WinLoadPointer(HWND_DESKTOP, hmodNonNLS, ID_SPY_POINTER);

      /*Ŀ*/
      /* Gather info about this client area's FRAME windows                 */
      /**/

      hwndFrame= WinQueryWindow(hwnd, QW_PARENT);
      hwndMenu = WinWindowFromID(hwndFrame, FID_MENU);

      /*Ŀ*/
      /* Initiate IPF support                                               */
      /**/

      if ( hwndIPF = InitializeIPF(hwndFrame,
                                   ID_AB_HELP,
                                   szHelpFile,
                                   Strings[IDS_HELP_TITLE_MAIN]) )
        MsgDisplayShouldEnableHELP();

      /*Ŀ*/
      /* Create PM SPY message listbox                                      */
      /*                                                                    */
      /* - set to non-porportional font                                     */
      /**/

      hwndMsgList=WinCreateWindow(hwnd,
                                  WC_LISTBOX,
                                  "classPMSPYlist",
                                  WS_VISIBLE   | WS_TABSTOP |
                                  LS_OWNERDRAW | LS_NOADJUSTPOS | LS_HORZSCROLL,
                                  0, 0, 0, 0,
                                  hwnd,
                                  HWND_TOP,
                                  ID_SPYLIST,
                                  NULL,
                                  NULL);

      /*Ŀ*/
      /* Set operational defaults                                           */
      /**/

      MaxNbrOfMessages   = atoi(Controls[CTL_MAX_MESSAGES]);
      MaxDeleteItems     = atoi(Controls[CTL_MAX_DELETEITEM]);

      /*Ŀ*/
      /* Our basic PM requirements are complete...time to register          */
      /* with our DLL resident support team                                 */
      /**/

      if ( SpyDllVersion() != DLLVERSION )   /* compatible .DLL and .EXE? */
      {
        MsgDisplay(HWND_DESKTOP,
                   Strings[IDS_TITLE],
                   Strings[IDS_FMT_LEVEL],
                   IDS_FMT_LEVEL,
                   MB_CUACRITICAL | MB_CANCEL,
                   DLLVERSION, SpyDllVersion());
        WinPostMsg(hwndFrame, WM_CLOSE, NULL, NULL);
        break;
      }

      /* can we register? */

      if ( (SpyInstance = SpyRegister(hwnd,                /* CLIENT gets data */
                                      PMSPY_MSG_FROM_HOOK))/* in this MSG */
           == SPY_BAD_SPYEE )
      {
        MsgDisplay(HWND_DESKTOP,
                   Strings[IDS_TITLE],
                   Strings[IDS_MSG_TOO_MANY_SPIES],
                   IDS_MSG_TOO_MANY_SPIES,
                   MB_CUACRITICAL| MB_CANCEL,
                   SpyQueryDllRegisterMax() );
        WinPostMsg(hwndFrame, WM_CLOSE, NULL, NULL);
        break;
      }

      /*Ŀ*/
      /* Get default or saved AGENT data                                    */
      /**/

      ReadSpyINI(hwndFrame, SpyInstance + 1, &iniAgent);

      WinSetPresParam(hwndMsgList,
                      PP_FONTNAMESIZE,
                      (ULONG) sizeof(iniAgent.szListFont)+1,  /* must include NULL! */
                      iniAgent.szListFont);

      /*Ŀ*/
      /* Process any command line arguments...                              */
      /**/

      { INT  j;

        for (j = 1;      /* Initialize (skip [0] which is program name */
             j < ArgC;   /* While more left to check */
             j++)        /* Iterate to next one */
        {
          strupr( ArgV[j] );

          if ( (strncmp(ArgV[j],
                        Strings[IDS_PARM_PROFILE],
                        strlen(Strings[IDS_PARM_PROFILE])) == 0) ||
               (strncmp(ArgV[j],
                        Strings[IDS_PARM_APPEND],
                        strlen(Strings[IDS_PARM_APPEND])) == 0) )
          {
            strcpy(szFileName, ArgV[j]+strlen(Strings[IDS_PARM_APPEND]));

            ProcessProfileFile(hwnd,
                               hwndFrame,
                               pSpyData,
                               strncmp( ArgV[j],
                                        Strings[IDS_PARM_APPEND],
                                        strlen(Strings[IDS_PARM_APPEND])) == 0
                                        ? FALSE : TRUE,
                               UsePassedFile,
                               szFileName);

          } else
          {
            /* unknown command line parameter will be ignored */
          }
        } /* endfor */

      }

      /*Ŀ*/
      /* finally, some final PM housekeeping                                */
      /**/
      WinSetWindowPos(hwndFrame,
                      HWND_TOP,
                      iniAgent.swpAgent.x,
                      iniAgent.swpAgent.y,
                      iniAgent.swpAgent.cx,
                      iniAgent.swpAgent.cy,
                      SWP_MOVE | SWP_SIZE | SWP_ZORDER | SWP_ACTIVATE | SWP_SHOW);


      sprintf(swcntrl.szSwtitle,
              Strings[IDS_FMT_TITLE],
              Strings[IDS_TITLE], SpyInstance + 1);

      UpdateSpyTitle(hwndFrame, tStandard);

      swcntrl.hwnd = hwndFrame;

      hSwitch = WinAddSwitchEntry((PSWCNTRL)&swcntrl);

      /* default to "Spying" on the window... */
      SpySetTargetIsWindow(SpyInstance, TRUE);
    }
    break;

    /*Ŀ*/
    /* We don't need to handle any other messages...                        */
    /*                                                                      */
    /* If this isn't an IPF message, let PM do it's default "thing"         */
    /**/
    default:
         if ( !HandleIPF(hwnd, msg, mp1, mp2, &wpResult) )
           wpResult = WinDefWindowProc(hwnd, msg, mp1, mp2);

  } /* endswitch */

  /*Ŀ*/
  /* Return result of this MSG's processing...                              */
  /**/

  return( wpResult );
}
